/**
 * ブラウザのコンテンツ表示管理
 */
var AppContent = (function () {
	// private field
	var css_list;
	var script_list;
	var set_script_list;
	var import_script_list;
	var first_script_list;
	var set_script_n = 0;
	var script_cache;
	var script_define_n;
	var readed_scp_n = 0;

	var app_onloads = [];
	var load_listeners = [];

	var axi_content = null;
	var headerArea = null;
	var footerArea = null;

	// window.onload, document.ready, addEventListener 対応用の正規表現
	var appOnloadCheckRegExp = /(?:window\.onload|(?:\$|jQuery)\((?:document)?\).ready|(?:\$|jQuery)\(function[\s\S]+}\))/;
	//var appAddEventListenerCheckRegExp = /window\.addEventListener\('load',[\s]*(function\([\s]*\)\{[\s\S]+}),[\s]*false\);?/;
	var appAddEventListenerCheckRegExp = /'load',[\s]*function\([\s]*\)\{/;

	// public Function/value
	return {
		onAppContentInit: function () {
			return onAppContentInit();
		},
		onLoadContent: function (url, param, add_page_history) {
			if (typeof add_page_history === 'undefined') add_page_history = true;
			return onLoadContent(url, param, add_page_history);
		},
		popupAddScript: function (before, after) {
			return popupAddScript(before, after);
		},
		preLoadResources: function () {
			return preLoadResources();
		},
		contentLoadErr: function (status, error) {
			return contentLoadErr(status, error);
		},
		header_data: {},		// ヘッダー
		footer_data: {},		// フッター
		popup_reserve: "",		// ポップアップURL
		last_url: "",		// 最終アクセスURL
		app_onloads: app_onloads,		// window.onload/jQuery(document).readyを後で実行するための一時領域
		load_listeners: load_listeners	// addEventListenerで追加されたloadイベントを後でremoveするための一時領域
	}

	/**
	 * 初期化
	 */
	function onAppContentInit() {
		PGLog.write("---------onAppContentInit called-------------");
		PGLog.writeWithTime("AppContent.onAppContentInit start");
		css_list = new Array();
		script_list = new Array();
		import_script_list = new Array();
		set_script_list = new Array();
		script_cache = new Array();
		first_script_list = new Array();

		script_define_n = document.scripts.length;

		var i = 0;
		for (i = 0; i < script_define_n; i++) {
			first_script_list[i] = document.scripts[i];
		}

		axi_content = document.getElementById("axi_content");
		axi_content.style.display = 'none';

		headerArea = document.getElementById("headerArea");
		footerArea = document.getElementById("footerArea");

		// ブラウザのコンテンツロード  
		registContentAdjustListener();
		PGLog.writeWithTime("AppContent.onAppContentInit end");
	}

	/**
	 * ブラウザのコンテンツ調整 のイベントリスナー
	 */
	function registContentAdjustListener(callbackFunc) {
		window.addEventListener("content_adjust_called", function () {
			PGLog.writeWithTime("AppContent.registContentAdjustListener start");

			PGLog.write("---------onAppContentInit -> registContentAdjustListener called");

			//ヘッダーフッター非表示
			headerArea.style.display = "none";
			footerArea.style.display = "none";
			Indicator.finishLoadPage();

			//ヘッダーフッターのデータが設定されていればここで表示
			if (AppContent.header_data != null) {
				headerArea = document.getElementById("headerArea");
				//hd.innerHTML = decodeURIComponent(AppContent.header_data.header);
				headerArea.innerHTML = AppContent.header_data.header;
				if (AppContent.header_data.visible) {
					jQuery('#headerArea').css('display', AppContent.header_data.visible);
				} else {
					jQuery('#headerArea').css('display', "none");
				}
				AppContent.header_data = null;
			}

			if (jQuery('#headerArea').css('display') == "none") {
				axi_content.style.top = 0;
			} else {
				axi_content.style.top = 46;
			}

			if (AppContent.footer_data != null) {
				footerArea = document.getElementById("footerArea");
				//ft.innerHTML = decodeURIComponent(AppContent.footer_data.footer);
				footerArea.innerHTML = AppContent.footer_data.footer;
				if (AppContent.footer_data.visible) {
					jQuery('#footerArea').css('display', AppContent.footer_data.visible);
				} else {
					jQuery('#footerArea').css('display', "none");
				}
				AppContent.footer_data = null;
			}

			axi_content.style.height = parseInt(window.innerHeight) + 1;
			axi_content.style.height = parseInt(window.innerHeight) - (parseInt(headerArea.clientHeight) );

			axi_content.style.display = 'block';

			//ポップアップ予約されていた場合
			if (AppContent.popup_reserve != null) {
				Popup.open_window(AppContent.popup_reserve);
				AppContent.popup_reserve = null;
			} else {
				Popup.disableAllMask();		// メニューダイアログはここを通らないので、全てのマスクを解除することにする
			}
			PGLog.writeWithTime("AppContent.registContentAdjustListener end");

		}, false);
	}

	/**
	 * コンテンツロード開始
	 */
	function onLoadContent(url, param, add_page_history) {
		PGLog.writeWithTime("AppContent.onLoadContent start");
		PGLog.write('---onLoadContent called.Url : ' + url);

		// インジケータ表示
		Indicator.beginLoadPage();

		App.enableBackKey = true;

		// Flash演出で登録されたリスナーを削除
		if (load_listeners.length > 0) {
			var cnt = load_listeners.length;
			for (var i = 0; i < cnt; i++) {
				if (AppContent.load_listeners.indexOf(i)) {
					window.removeEventListener('load', AppContent.load_listeners[i], false);
				}
			}
			AppContent.load_listeners = [];
			load_listeners = [];
		}
		// global で重たい変数も初期化しておく
		if (typeof AN !== "undefined") AN = {};		// sencha オブジェクト
		if (typeof swiffyobject !== "undefined") swiffyobject = {};		// swiffy オブジェクト

		PGLog.write('コンテンツロードの送信パラメータ  : ' + param);

		var is_first_login = PGLocalStorage.getVal('is_first_login');
		PGLog.write("is_first_login : " + is_first_login);

		var device_uuid = PGLocalStorage.getVal('device_uuid');
		var device_type = PGLocalStorage.getVal('device_type');
		var post_data = {};

		if ((typeof param !== 'undefined') && (param !== null)) {
			post_data = param;
		} else {
			post_data = {
				'is_yapp': 1,
				'device_uuid': device_uuid,
				'device_type': device_type,
				'is_first_login': is_first_login
			};
		}

		//最後にリクエストしたURLを記憶
		AppContent.last_url = url;

		//ページ履歴を保存
		if (add_page_history) {
			AppPageHistory.pushPageHistory();
		}
		ServerInquire.postDataWithDataType(url, post_data, "html", loadContentComplete, contentLoadErr);
		PGLog.writeWithTime("AppContent.onLoadContent end");
	}

	/**
	 * コンテンツロード完了
	 */
	function loadContentComplete(data) {
		PGLog.writeWithTime("AppContent.loadContentComplete start");
		PGLog.write('---loadContentComplete called------');

		// ローカルサーバ死活監視
		pGLocalHttpd.getDataBeforeCheckServer(data, _loadContentComplete);
		PGLog.writeWithTime("AppContent.loadContentComplete end");
	}

	/**
	 * コンテンツロード完了
	 */
	function _loadContentComplete(data) {
		PGLog.writeWithTime("AppContent._loadContentComplete start");
		PGLog.write('---_loadContentComplete called------');

		//読み込んだスクリプトの中に重複しているものを開放
		removeScript();

		//読み込んだCSSの中に重複しているものを開放
		removeCSS();

		//ページセット
		axi_content.innerHTML = data;
		axi_content.style.height = "100%";

		//これで呼んできたページのスクリプト動かせそう
		var scp_n = document.scripts.length;
		var import_scp_cnt = 0;
		var i;

		var scp_cnt = 0;
		app_onloads = [];
		this.app_onloads = [];
		load_listeners = [];
		this.load_listeners = [];

		import_script_list.length = 0;

		for (i = 0; i < scp_n; i++) {
			if (document.scripts[i].text) {
				script_list[scp_cnt] = document.scripts[i];
				scp_cnt++;
			} else if (document.scripts[i].src && (document.scripts[i].src.indexOf("file://") < 0)) {
				//外部jsを読む
				//if(document.scripts[i].src.indexOf("jquery") < 0)		// ゲームサーバで対応するので判定しない
				//{
				import_script_list[import_scp_cnt] = document.scripts[i];
				import_scp_cnt++;
				//}
			}
		}

		// 音声再生
		soundManager.execPlayScene(AppContent.last_url);

		if (import_script_list.length > 0) {
			//外部jsを読み込む
			readed_scp_n = 0;

			scriptImport(null);
		} else {
			//scriptを追加する
			addScript();
		}
		PGLog.writeWithTime("AppContent._loadContentComplete end");
	}

	/**
	 * スクリプトを削除
	 */
	function removeScript() {
		PGLog.writeWithTime("AppContent.removeScript start");
		PGLog.write('---removeScript called------');

		var i;
		var len = set_script_list.length;
		var rem_list;

		for (i = 0; i < len; i++) {
			if (set_script_list[i]) {
				try {
					document.body.removeChild(set_script_list[i]);
				} catch (e) {
					PGLog.write(e.message);
				}
				set_script_list[i] = null;
			}
		}

		script_list.length = 0;
		set_script_list.length = 0;
		set_script_n = 0;
		PGLog.writeWithTime("AppContent.removeScript end");
	}

	/**
	 * CSSを削除
	 */
	function removeCSS() {
		PGLog.writeWithTime("AppContent.removeCSS start");
		PGLog.write('---removeCSS called------');

		var h = document.getElementsByTagName('head')[0];
		var key;
		for (key in css_list) {
			if (css_list[key]) {
				h.removeChild(css_list[key]);
				css_list[key] = null;
			}
		}
		PGLog.writeWithTime("AppContent.removeCSS end");
	}

	/**
	 * 外部jsを読み込む
	 */
	function scriptImport(doc) {
		PGLog.writeWithTime("AppContent.scriptImport start");
		PGLog.write('---scriptImport called------');

		var i = 1;
		if (doc) {
			// window.onload/document.ready は後で実行する
			var app_onloads_index = app_onloads.length < 1 ? 0 : app_onloads.length;
			var load_listeners_index = 0;
			var matchRegex = null;
			var matches = [];
			var tmp;
			var loopMax = 10;
			if (doc.match(appOnloadCheckRegExp)) {
				try {
					if (doc.match(/window\.onload/)) {
						// window.onload = function(){}
						matchRegex = /window\.onload *?= *?(function[\s\S]+})/;
					} else {
						if (doc.match(/(?:\$|jQuery)\((?:document)?\).ready/)) {
							matchRegex = /(?:\$|jQuery)\((?:document)?\).ready\(([\s\S]+})\);?/;
						} else {
							matchRegex = /(?:\$|jQuery)\((function[\s\S]+})\)/;
						}
					}

					var loopCount = 0;
					while ((matches = doc.match(matchRegex)) !== null) {
						tmp = "AppContent.app_onloads[" + app_onloads_index + "] = " + matches[1] + ";\n";
						tmp = tmp + "AppContent.app_onloads[" + app_onloads_index + "]()";
						app_onloads[app_onloads_index] = tmp;
						app_onloads_index++;
						doc = doc.replace(matchRegex, "");
						loopCount++;
						if (loopCount > loopMax) break;		// 保険処理
					}
					//doc = doc.replace(replaceRegex, "");
				} catch (e) {
					PGLog.write(e.message);
				}
			} else if (doc.match(appAddEventListenerCheckRegExp)) {
				//loadイベントを使用している場合、リスナー関数を取得する
				load_listeners_index = load_listeners.length < 1 ? 0 : load_listeners_index.length;
				doc = doc.replace(appAddEventListenerCheckRegExp, "'load', AppContent.load_listeners[" + load_listeners_index + "] = function(){");
				load_listeners[load_listeners_index] = load_listeners_index;
			}
		}

		if (doc) {
			import_script_list[readed_scp_n].text = doc;
			if (script_cache[import_script_list[readed_scp_n].src]) {
			} else {
				script_cache[import_script_list[readed_scp_n].src] = doc;
			}
			readed_scp_n++;
		}

		if (readed_scp_n < import_script_list.length) {
			// Swiffy 演出対応
			if (import_script_list[readed_scp_n].src.match(/runtime[\d.]*?\.js/)) {
				App.enableBackKey = false;
			}

			//キャッシュにhitしたら読みに行かない
			if (script_cache[import_script_list[readed_scp_n].src]) {
				scriptImport(script_cache[import_script_list[readed_scp_n].src]);
			} else {
				var post_data = {
					'is_yapp': 1,
					'device_uuid': PGLocalStorage.getVal('device_uuid'),
					'device_type': PGLocalStorage.getVal('device_type'),
					'is_first_login': PGLocalStorage.getVal('is_first_login')
				};
				ServerInquire.postDataWithDataType(import_script_list[readed_scp_n].src, post_data, "text", scriptImport, contentLoadErr);
			}
		} else {
			//読めたであろうスクリプトを反映
			// TODO innerHTML変更をループの外にして、リフロー回数を減らす
			for (i = 0; i < readed_scp_n; i++) {
				try {
					var scp = document.createElement("script");
					scp.innerHTML = import_script_list[i].text;

					document.body.appendChild(scp);
					set_script_list[set_script_n] = scp;
					set_script_n++;
				} catch (e) {
					PGLog("scriptImport Error:" + e.message());
				}
			}
			addScript();
		}
		PGLog.writeWithTime("AppContent.scriptImport end");
	}


	/**
	 * scriptを追加する
	 */
	function addScript() {
		PGLog.writeWithTime("AppContent.addScript start");
		PGLog.write('---addScript called------');

		var i = 0;
		var j = 0;
		var scp_cnt = script_list.length;
		var app_onloads_index = 0;
		var load_listeners_index = 0;
		var scp = null;

		var tmp_texts = [];
		var tmp_text = "";
		var readyRegex = null;

		// TODO innerHTML変更をループの外にして、リフロー回数を減らす
		for (i = 0; i < scp_cnt; i++) {
			if (checkScript(script_list[i])) {
				if (script_list[i].text) {
					var scp_txt = script_list[i].text;

					if (scp_txt.match(appAddEventListenerCheckRegExp)) {
						//loadイベントを使用している場合、リスナー関数を取得する
						load_listeners_index = load_listeners.length < 1 ? 0 : load_listeners_index.length;
						scp_txt = scp_txt.replace(appAddEventListenerCheckRegExp, "'load', AppContent.load_listeners[" + load_listeners_index + "] = function(){");
						load_listeners[load_listeners_index] = load_listeners_index;
					}

					if (scp_txt.match(appOnloadCheckRegExp)) {
						// 事故防止のためラインコメントとブロックコメントを削除
						scp_txt = scp_txt.replace(/\/\*[\s\S]\*\//, "");

						tmp_texts = scp_txt.split(/\r\n|\r|\n/);
						scp_txt = "";
						for (j = 0; j < tmp_texts.length; j++) {
							tmp_text = tmp_texts[j];
							tmp_text = tmp_text.replace(/^\s*\/\/.*/, "");
							tmp_text = tmp_text.replace(/;\s*\/\/.*/, ";");
							scp_txt += tmp_text + "\n";
						}
					}

					// window.onload/jQuery().ready は後で実行する
					if (scp_txt.match(appOnloadCheckRegExp)) {
						try {
							app_onloads_index = app_onloads.length < 1 ? 0 : app_onloads.length;
							if (scp_txt.match(/window\.onload/)) {
								scp_txt = scp_txt.replace("window.onload", "AppContent.app_onloads[" + app_onloads_index + "]");
							} else {
								readyRegex = null;
								if (scp_txt.match(/(?:\$|jQuery)\((?:document)?\).ready/)) {
									readyRegex = /(?:\$|jQuery)\((?:document)?\).ready\(([\s\S]+})\);?/;
								} else {
									readyRegex = /(?:\$|jQuery)\((function[\s\S]+})\)/;
								}
								scp_txt = scp_txt.replace(readyRegex, "AppContent.app_onloads[" + app_onloads_index + "] = $1;");
							}
							scp_txt = scp_txt + "AppContent.app_onloads[" + app_onloads_index + "]();";
							app_onloads[app_onloads_index] = document.createElement("script");
							app_onloads[app_onloads_index].innerHTML = scp_txt;
						} catch (e) {
							PGLog.write(e.message);
						}
					} else {
						try {
							scp = document.createElement("script");
							scp.innerHTML = scp_txt;

							document.body.appendChild(scp);
							set_script_list[set_script_n] = scp;
							set_script_n++;
						} catch (e) {
							PGLog.write(e.message);
						}
					}
				} else if (script_list[i].src && (script_list[i].src.indexOf("file://") < 0)) {
					//もうここには来ないはず
					scp = document.createElement("script");
					scp.src = script_list[i].src;
					axi_content.appendChild(scp);
					set_script_list[set_script_n] = scp;
					set_script_n++;
				}
			}
		}

		//divのスクロールをトップへ
		axi_content.scrollTop = 0;
		// ページ内リンクがあるならスクロール位置をずらす
		var window_hash_target = '';
		if (typeof AppContent.last_url !== 'undefined' && AppContent.last_url != '') {
			if (AppContent.last_url.match(/#.+?/)) {
				window_hash_target = (AppContent.last_url.match(/#(.+?)$/) || [])[1];
				window_hash_target = rawurldecode(window_hash_target);
				location.hash = window_hash_target;
			}
		}

		if (app_onloads.length > 0) {
			for (i = 0; i < app_onloads.length; i++) {
				try {
					document.body.appendChild(app_onloads[i]);
					set_script_list[set_script_n] = app_onloads[i];
					set_script_n++;
				} catch (e) {
				}
			}
		}

		// PGLog.write("ページ読み込み完了!!");

		//ロードイベントを発行 -> アプリ用読み込み通知
		var customEvent = document.createEvent("HTMLEvents");
		customEvent.initEvent("content_adjust_called", true, false);

		//fire!!
		window.dispatchEvent(customEvent);

		//sencha用読み込み通知
		var swfEvent = document.createEvent("HTMLEvents");
		swfEvent.initEvent("load", true, false);

		//fire!!
		window.dispatchEvent(swfEvent);
		PGLog.writeWithTime("AppContent.addScript end");
	}

	function checkScript(scp) {
		PGLog.write('---checkScript called------');

		var i;
		for (i = 0; i < first_script_list.length; i++) {
			if ((scp.src == first_script_list[i].src) && (scp.text == first_script_list[i].text)) {
				return false;
			}
		}

		return true;
	}

	/**
	 * 現在読み込まれているスクリプトと適用したスクリプトの差分から、スクリプトを追加する
	 */
	function popupAddScript(before, after) {
		PGLog.writeWithTime("AppContent.popupAddScript start");
		var scp_n;
		var i, j;
		var add_scp_list;
		var add_scp_n = 0;
		var is_set;

		scp_n = after.length;
		add_scp_list = new Array();

		for (i = 0; i < scp_n; i++) {
			is_set = 0;

			for (j = 0; j < before.length; j++) {
				if (after[i] === before[j]) {
					is_set = 1;
					break;
				}
			}

			if (is_set == 0) {
				add_scp_list[add_scp_n] = after[i];
				add_scp_n++;
			}
		}

		// TODO innerHTML変更をループの外にして、リフロー回数を減らす
		for (i = 0; i < add_scp_n; i++) {
			var scp = document.createElement("script");
			scp.innerHTML = add_scp_list[i].text;
			document.body.appendChild(scp);
			set_script_list[set_script_n] = scp;
			set_script_n++;
		}

		PGLog.writeWithTime("AppContent.popupAddScript end");
	}

	/**
	 * コンテンツロードエラー
	 */
	function contentLoadErr(status, error) {
		if (typeof status !== "undefined" && typeof error !== "undefined") PGLog.write("contentLoadErr: " + status + " : " + error);
		PG_Dialog.dialogGetContentFail();
	}

	/**
	 * リソースのプレロード
	 * @returns {boolean}
	 */
	function preLoadResources() {
		PGLog.writeWithTime("AppContent.preLoadResources start");
		var isReaded = appConfig.getConfig('isPreLoadResources', false);
		if (isReaded != 1) {
			pGLocalHttpd.isRunning(
				function (msg) {
					// Webサーバからロードするリストを取得
					//var post_data = {};		// 認証不要（ログイン前に実行させたい）
					var post_data = {
						'is_yapp': 1,
						'device_uuid': PGLocalStorage.getVal('device_uuid'),
						'device_type': PGLocalStorage.getVal('device_type'),
						'is_first_login': PGLocalStorage.getVal('is_first_login')
					};		// 新規or既存判別をするために渡す

					var axi_server_url = PGLocalStorage.getVal('axi_server_url');
					var request_url = axi_server_url + "/YappHomes/get_pre_load_resources";
					PGLog.write("start pre load: " + request_url);

					// Bridgeのユーザー情報取得
					ServerInquire.postData(request_url, post_data,
						function (data) {
							if (data.response === 'success') {
								var resources = data.resources;
								pGLocalHttpd.loadResources(resources);
							}
						},
						function (status, error) {
							PGLog.write("POST DATA ERROR.Status:" + status + "/Err:" + error + "/Url:" + request_url);
						}
					);
					appConfig.setConfig('isPreLoadResources', 1, false);
				}
			);
		}
		PGLog.writeWithTime("AppContent.preLoadResources end");
		return true;
	}

})();

