/*
 *	ログインまわりの処理
 */
var AppLogin = (function() {
	// local value
	var reloginCountMax = 5;		// 再ログイン遷移の最大回数定義
	var reloginCount = 0;
	var isMaintenance = false;		// メンテ状態の一時領域

	// public Function
	return {
		loginEntry: function () {
			return loginEntry();
		},
		getNativeToken: function (nativeToken) {
			return getNativeToken(nativeToken);
		},
		checkIfFirstTimeLogin: function (device_uuid_key) {
			return checkIfFirstTimeLogin(device_uuid_key);
		},
		recordLoginInfo: function (login_user_name,invite_code) {
			return recordLoginInfo(login_user_name,invite_code);
		},
		getUUIDFromSharedPreferences: function (device_uuid) {
			return getUUIDFromSharedPreferences(device_uuid);
		},
		setGatewayNativeSessionCreate: function (nativeTokenInfoAndUUIDStr) {
			return setGatewayNativeSessionCreate(nativeTokenInfoAndUUIDStr);
		},
		updateNativeSession: function (bridgeNativeSession) {
			return updateNativeSession(bridgeNativeSession);
		},
		securityReinforceCallback: function (callback) {
			return securityReinforceCallback(callback);
		},
		peopleLoginCallback: function (callback) {
			return peopleLoginCallback(callback);
		},
		getBridgeUserInfo:function() {
			return getBridgeUserInfo();
		},
		continueLoginAfterGetBridgeUserInfo:function(callback) {
			return continueLoginAfterGetBridgeUserInfo(callback);
		},
		startGameAfterBridgeLogin:function(){
			return startGameAfterBridgeLogin();
		},
		renderPageContent:function(){
			return renderPageContent();
		},
		changeConnectEnv:function(env_name){
			return changeConnectEnv(env_name);
		},
		initUserData:function(){
			return initUserData();
		}
	}
	
	/**
	 * ログイン処理開始
	 */
	function loginEntry(){
		PGLog.writeWithTime("AppLogin.loginEntry start");
		PGLocalStorage.setVal('is_first_login',0);
		var device_uuid = PGLocalStorage.getVal('device_uuid');
		reloginCount = 0;
		
		if(((typeof device_uuid !== 'undefined') && (device_uuid !== null)) && (typeof param !== 'undefined') && (param.hasOwnProperty("boot_url")) && (param !== null)){
			// 起動URLが指定されていたら、BRIDGE認証スキップ
			PGLog.write("起動URLが指定されている。BRIDGE認証スキップ");
			renderPageContent();
			
		}else{
			// SharedPreferencesからUUIDを取り出し。その後、checkIfFirstTimeLoginにコールバックする
			PGConfig.findKeyInSharedPreferences('device_uuid',AppLogin.checkIfFirstTimeLogin);
		}
		PGLog.writeWithTime("AppLogin.loginEntry end");
	}
	
	/**
	 * はじめてアプリ起動するかどうかチェックを行う
	 */
	function checkIfFirstTimeLogin(device_uuid_key)
	{
		PGLog.writeWithTime("AppLogin.checkIfFirstTimeLogin start");
		PGLog.write("SharedPreferencesにUUIDを探す:"+device_uuid_key);
		
		if(device_uuid_key === "device_uuid"){
			// SharedPreferencesにはUUIDが存在している
			PGLog.write("SharedPreferencesにはUUIDが存在している");
	 		PGLocalStorage.setVal('is_first_login',0);
	 		
			// SharedPreferencesにUUID存在 → アプリ初回起動ではない  -> SharedPreferencesからUUID取得 。その後、getUUIDFromSharedPreferencesにコールバックする
			PGConfig.getStr('device_uuid',AppLogin.getUUIDFromSharedPreferences);
			
		}else{
			// 初回起動です
			PGLog.write("SharedPreferencesにもUUIDが存在していない。アプリの初回起動です");
			PGLocalStorage.setVal('is_first_login',1);
			
			// Track application install event
			PGLog.write("********** TrackAppInstallEvent ************");
			MetapsAnalytics.trackAppInstallEvent();
			
			// セキュリティ強化対策
			nativeTokenCreate(setGatewayNativeSessionCreate,null);
		}
		PGLog.writeWithTime("AppLogin.checkIfFirstTimeLogin end");
	}
	
	/**
	 * SharedPreferencesからUUIDを取り出し、ローカルストレージにUUID記録
	 */
	function getUUIDFromSharedPreferences(device_uuid)
	{
		PGLog.writeWithTime("AppLogin.getUUIDFromSharedPreferences start");
		PGLog.write("getUUIDFromSharedPreferences:"+device_uuid);
		
		// 文字列の長さをチェック
		var device_uuid_length = device_uuid.length;
		
		if(device_uuid_length == 36){ // device_uuidの長さは36文字
			// SharedPreferencesから取り出すdevice_uuidをローカルストレージに保存する
			PGLocalStorage.setVal('device_uuid',device_uuid);
		
			// セキュリティ強化対策
			nativeTokenCreate(setGatewayNativeSessionCreate,device_uuid);
		}else{
			if (!checkReloginCount()) return;
			AppLogin.loginEntry();
			
			// インジケータ非表示
	    	Indicator.finishLoadPage();
		}
		PGLog.writeWithTime("AppLogin.getUUIDFromSharedPreferences end");
	}
	
	/** 
	 * セキュリティ強化対策
	 */
	function nativeTokenCreate(callbackFunc,device_uuid){
		PGLog.writeWithTime("AppLogin.nativeTokenCreate start");
		if((typeof device_uuid !== 'undefined') && device_uuid !== null){
			PGLog.write("nativeTokenCreate.device_uuid : " + device_uuid);
			
			var post_data = {'device_uuid':device_uuid};
		}else{
			PGLog.write("nativeTokenCreate.device_uuid not exist");
			
			var post_data = {};
		}
		
		var axi_server_url = PGLocalStorage.getVal('axi_server_url');
		var request_url = axi_server_url + "/YappHomes/getNativeTokenAndUUID";
		
		ServerInquire.postData(request_url,post_data,callbackFunc,postErrFunc);
		PGLog.writeWithTime("AppLogin.nativeTokenCreate end");
	}
	
	/**
	 * ネイティブセッションを作成する
	 * BRDIGE API実装
	 */
	function setGatewayNativeSessionCreate(nativeTokenInfoAndUUID){
		PGLog.writeWithTime("AppLogin.setGatewayNativeSessionCreate start");
		if((typeof nativeTokenInfoAndUUID !== 'undefined') && (nativeTokenInfoAndUUID !== null)){
			
			var device_uuid = null;
			var nativeToken = null;
			var device_uuid_server = null;
			
			if(typeof nativeTokenInfoAndUUID === 'object'){
				nativeToken = escapeSpecialSymbol(nativeTokenInfoAndUUID.nativeToken);
				device_uuid_server = nativeTokenInfoAndUUID.device_uuid;
				if (typeof nativeTokenInfoAndUUID.isMaintenance !== "undefined" && nativeTokenInfoAndUUID.isMaintenance == 1) isMaintenance = true;
			} else {
				var nativeTokenInfoAndUUIDJson;
				try
				{
					nativeTokenInfoAndUUIDJson = JSON.parse(nativeTokenInfoAndUUID);
				} catch (e) {
					// BRIDGE APIでエラーが起きた場合
					PGLog.write("nativeTokenInfoAndUUID is not json:" + nativeTokenInfoAndUUID);
					if (!checkReloginCount()) return;
					AppLogin.loginEntry();
					return;
				}

				nativeToken = escapeSpecialSymbol(nativeTokenInfoAndUUIDJson.nativeToken);
				device_uuid_server = nativeTokenInfoAndUUIDJson.device_uuid;
				if (typeof nativeTokenInfoAndUUIDJson.isMaintenance !== "undefined" && nativeTokenInfoAndUUIDJson.isMaintenance == 1) isMaintenance = true;
			}
			
			// サーバーからのUUIDと比較
			var device_uuid_client = PGLocalStorage.getVal('device_uuid'); // デバイス上のUUID
			if((typeof device_uuid_client !== 'undefined') && (device_uuid_client !== null)){
				// 端末にdevice_uuidが存在していない（サーバーが発行してもらうdevice_uuidを使う）
				device_uuid = device_uuid_client;
			}else{
				device_uuid = device_uuid_server;
			}
			
			// ローカルストレージにネイティブトークンを保存する
			PGLocalStorage.setVal('bridge_native_token',nativeToken);
			
			// 文字列の長さをチェック
			var device_uuid_length = device_uuid.length;
			
			if(device_uuid_length == 36){ // device_uuidの長さは36文字
				PGLocalStorage.setVal('device_uuid',device_uuid);
			}
			
			// Bridge webサーバーURL
			var bridge_api_endpoint = PGLocalStorage.getVal('bridge_api_endpoint');
			
			// LocalStorageに保存
			var device_platform = device.platform;
			if(device_platform == 'Android'){
				device_type = 2;
			}else{
				device_type = 1;
			}
			
			// デバイスタイプもローカルストレージに保存する
			PGLocalStorage.setVal('device_type',device_type);
			 	
			PGLog.write("device_uuid :" + device_uuid);
			PGLog.write("device_type :" + device_type);
			PGLog.write("nativeToken :" + nativeToken);

			var post_data = {
								'UUID':device_uuid,
			                  	'deviceType':device_type,
								'nativeToken':nativeToken
							};
							
		    PGLog.write('https: ' + bridge_api_endpoint);
		    PGLog.write('送信nativeToken: ' + nativeToken);

			if (isMaintenance) {
				PGLog.write("Now maintenance. skip login method");
				renderPageContent();
				return;
			}
			
			// Bridgeサーバーに直接問い合わせ
			ServerInquire.postDataBySSL(bridge_api_endpoint,post_data,AppLogin.updateNativeSession,postErrFunc);
		}else{
			// エラーの場合は再度接続
			if (!checkReloginCount()) return;
			AppLogin.loginEntry();
			
			// インジケータ非表示
	    	Indicator.finishLoadPage();
		}
		PGLog.writeWithTime("AppLogin.setGatewayNativeSessionCreate end");
		
	}
	
	/**
	 * Bridgeにセッション更新してもらうようサーバーに依頼する
	 * BridgeサーバーからnativeSessionId+sharedSecurityKey受け取る
	 */
	function updateNativeSession(bridgeNativeSession){
		PGLog.writeWithTime("AppLogin.updateNativeSession start");
		PGLog.write("updateNativeSession called.BridgeNativeSession : " + bridgeNativeSession);
		
		if((typeof bridgeNativeSession !== 'undefined') && (bridgeNativeSession !== null)){
			var nativeSessionId = null;
		
			if(typeof bridgeNativeSession === 'object'){
				nativeSessionId = escapeSpecialSymbol(bridgeNativeSession.nativeSessionId);
			} else {
				var bridgeNativeSessionJson;
				try
				{
					bridgeNativeSessionJson = JSON.parse(bridgeNativeSession);
				} catch (e) {
					PGLog.write("bridgeNativeSessionJson is not json:" + bridgeNativeSessionJson);
					if (!checkReloginCount()) return;
					AppLogin.loginEntry();
					return;
				}
				
				nativeSessionId = escapeSpecialSymbol(bridgeNativeSessionJson.nativeSessionId);
				
			}
				
			var device_uuid = PGLocalStorage.getVal('device_uuid');
			var device_type = PGLocalStorage.getVal('device_type');
			var is_first_login = PGLocalStorage.getVal('is_first_login');
			
			PGLog.write("device_uuid :" + device_uuid);
			PGLog.write("device_type :" + device_type);
			PGLog.write("nativeSessionId :" + nativeSessionId);
			
			var post_data = {	'is_yapp':1,
				                'device_uuid':device_uuid,
			                  	'device_type':device_type,
								'native_session_id':nativeSessionId,
								'is_first_login':is_first_login
							};
							
			// ゲームサーバーにnativeSessionIdを渡す
			var axi_server_url = PGLocalStorage.getVal('axi_server_url');
			var request_url = axi_server_url + "/YappHomes/updateNativeSession";
		
			ServerInquire.postData(request_url,post_data,AppLogin.securityReinforceCallback,postErrFunc);
		}else{
			// エラーの場合は再度行う
			if (!checkReloginCount()) return;
			AppLogin.loginEntry();
			
			// インジケータ非表示
	    	Indicator.finishLoadPage();
		}
		PGLog.writeWithTime("AppLogin.updateNativeSession end");
	}
	
	/**
	 * セキュリティ強化のコールバック
	 */
	function securityReinforceCallback(callback){
		PGLog.writeWithTime("AppLogin.securityReinforceCallback start");
	    PGLog.write('securityReinforceCallback called.SecurityReinforceCallback: ' + callback);
	    
	    // インジケータ非表示
	    Indicator.finishLoadPage();
	    
		if((typeof callback !== 'undefined') && (callback === 'success')){
			renderPageContent();
		}else{
			// エラー発生の場合、ログインのエントリーに戻る
			if (!checkReloginCount()) return;
			AppLogin.loginEntry();
		}
		PGLog.writeWithTime("AppLogin.securityReinforceCallback end");
	}
	
	/**
	 * 入力されたユーザ名や招待コード登録 (ユーザー名登録・招待コード登録ページからの遷移先)
	 */
	function recordLoginInfo(login_user_name,invite_code){
		PGLog.writeWithTime("AppLogin.recordLoginInfo start");
		// ローカルストレージに名前を保存する
		if((typeof login_user_name !== 'undefined') && (login_user_name !== null)){
			
			PGLog.write("入力されたプレイヤ名 : " + login_user_name);
		
			var device_uuid = PGLocalStorage.getVal('device_uuid');
			var device_type = PGLocalStorage.getVal('device_type');
			var is_first_login = PGLocalStorage.getVal('is_first_login');
			
			var post_data = {	'is_yapp':1,
				                'device_uuid':device_uuid,
			                  	'device_type':device_type,
								'is_first_login':is_first_login,
								'login_user_name':login_user_name,
								'invite_code':invite_code
							};
							
			// ゲームサーバーにnativeSessionIdを渡す
			var axi_server_url = PGLocalStorage.getVal('axi_server_url');
			var request_url = axi_server_url + "/YappEntries/recordLoginInfo";
			
			ServerInquire.postData(request_url,post_data,AppLogin.peopleLoginCallback,postErrFunc);
				
		}else{
			navigator.notification.alert("プレイヤ名の登録が必要です");
		}
		PGLog.writeWithTime("AppLogin.recordLoginInfo end");
	}
	
	/**
	 * ユーザー情報登録や招待コード入力が終わったあと、こちらに来る
	 */
	function peopleLoginCallback(callback){
		PGLog.writeWithTime("AppLogin.peopleLoginCallback start");
		var response = null;
		var err_type = null;
		var status = null;
		
		if(typeof callback === 'object'){
			response = callback.response;
			err_type = callback.type;
			status = callback.status;
			
		} else {
			var callbackJson;
			try
			{
				callbackJson = JSON.parse(callback);
			} catch (e) {
				PGLog.write("callbackJson is not json:" + callback);
				return;
			}
			
			response = callbackJson.response;
			err_type = callbackJson.type;
			status = callbackJson.status;
			
		}
		
		PGLog.write(response);
		PGLog.write(err_type);
		PGLog.write(status);
		
		if(response == 'success' && status == 'ok'){
			// ユーザー名登録完了状態
			PGLocalStorage.setVal('is_first_login',0);
			
			// ローカルストレージからUUID取り出し
			var device_uuid = PGLocalStorage.getVal('device_uuid');
			
			// 文字列の長さをチェック
			var device_uuid_length = device_uuid.length;
		
			if(device_uuid_length == 36){ // device_uuidの長さは36文字
				// Bridge認証に成功したのでUUIDを保存する（SharedPreferencesに保存）
				PGConfig.putStr('device_uuid',device_uuid);
				PGLog.write("Put UUID to SharedPreferences.UUID : "+device_uuid);
			}else{
				PGLog.write("不正なdevice_uuid!!!");
				if (!checkReloginCount()) return;
				AppLogin.loginEntry();
			}
			renderPageContent();
			
			// Push通知
			PGPushNotification.initPushNotification();
			
		} else {
			YAppHome.recordUserNameError(err_type);
		}
		PGLog.writeWithTime("AppLogin.peopleLoginCallback end");
		
	}
	
	/**
	 * データー引き継ぎでBridgeログインのあとこちらに来る
	 */
	function startGameAfterBridgeLogin(){
		PGLog.writeWithTime("AppLogin.startGameAfterBridgeLogin start");
		// ユーザー名登録完了状態
		PGLocalStorage.setVal('is_first_login',0);
		
		// ローカルストレージからUUID取り出し
		var device_uuid = PGLocalStorage.getVal('device_uuid');
		
		// 文字列の長さをチェック
		var device_uuid_length = device_uuid.length;
		
		if(device_uuid_length == 36){ // device_uuidの長さは36文字
			// Bridge認証に成功したのでUUIDを保存する（SharedPreferencesに保存）
			PGConfig.putStr('device_uuid',device_uuid);
			PGLog.write("Put UUID to SharedPreferences.UUID : "+device_uuid);
		}else{
			PGLog.write("不正なdevice_uuid!!!");
			if (!checkReloginCount()) return;
			AppLogin.loginEntry();
		}

		renderPageContent();
		PGLog.writeWithTime("AppLogin.startGameAfterBridgeLogin end");
	}
	
	
	/**
	 * ブラウザのコンテンツ表示
	 */
	function renderPageContent(){
		PGLog.writeWithTime("AppLogin.renderPageContent start");
			// ログイン状態管理フラグを取り出し
			var is_first_login = PGLocalStorage.getVal('is_first_login');
			
			if((typeof param !== 'undefined') && (param.hasOwnProperty("boot_url")) && (param !== null)){
				// Flash演出後、アプリに戻る導線
				PGLog.write("アプリに戻りました。Boot url : "+param["boot_url"]);
				
				// コンテンツ表示
			    AppContent.onLoadContent(rawurldecode(param["boot_url"]));
			}else{
				var axi_server_url = PGLocalStorage.getVal('axi_server_url');
				
				if(is_first_login == 1){
					// 利用規約同意・名前登録ページ表示
					axi_server_url = axi_server_url + "/yapp_homes";
					AppContent.onLoadContent(axi_server_url,null);
				}else{
					// アプリの通常起動
					PGLog.write("アプリの通常起動");
					// クライアントバージョンを送信
					var client_version = PGLocalStorage.getVal('client_version');
					axi_server_url += '?client_version=' + client_version;
					AppContent.onLoadContent(axi_server_url,null);
				}
			}
		PGLog.writeWithTime("AppLogin.renderPageContent end");
	}
	
	/**
	 * ゲーム開始時の名前入力でBridgeに登録した情報取得
	 */
	function getBridgeUserInfo(){
		PGLog.writeWithTime("AppLogin.getBridgeUserInfo start");
		var device_uuid = PGLocalStorage.getVal('device_uuid');
		var device_type = PGLocalStorage.getVal('device_type');
		
		var post_data = {
						'device_uuid':device_uuid,
		                'device_type':device_type,
						};
		var axi_server_url = PGLocalStorage.getVal('axi_server_url');
		var request_url = axi_server_url + "/YappHomes/getBridgeUserInfo";
		
		// Bridgeのユーザー情報取得
		ServerInquire.postData(request_url,post_data,AppLogin.continueLoginAfterGetBridgeUserInfo,postErrFunc);
		PGLog.writeWithTime("AppLogin.getBridgeUserInfo end");
	}
	
	/**
	 * ゲーム開始時Bridgeに登録した情報を取得したあと、ログイン処理継続
	 */
	function continueLoginAfterGetBridgeUserInfo(callback){
		PGLog.writeWithTime("AppLogin.continueLoginAfterGetBridgeUserInfo start");
		if(typeof callback !== 'undefined'){
			var response = callback.response;
			
			if(response === 'success'){
				var nickName = callback.nickName;
				
				// ローカルストレージに保存する
				PGLocalStorage.setVal('user_nick_name',nickName);
				
				// ログイン処理継続
				YAppHome.startGameAfterLogin();
			}
		}
		PGLog.writeWithTime("AppLogin.continueLoginAfterGetBridgeUserInfo end");
	}

	/**
	 * 再ログイン回数チェック
	 *
	 * @returns {boolean}
	 */
	function checkReloginCount()
	{
		reloginCount++;
		if (reloginCountMax < reloginCount) {
			postErrFunc();		// ログインエラーダイアログ
			return false;
		}
		return true;
	}
	
	/**
	 * データー送信エラー
	 */
	function postErrFunc(status, error)
	{
		if (typeof status !== "undefined" && typeof error !== "undefined") PGLog.write("POST DATA ERROR.Status:" + status + "/Err:" + error);
		PG_Dialog.dialogLoginFail();
	}
	
	/**
	 * 特殊文字列をエスケープする
	 */
	function escapeSpecialSymbol(str) {
		PGLog.write("Before escapeSpecialSymbol  : " + str);
		str = str.replace(/\\n/g, "\\n");
		str = str.replace(/\\'/g, "\\'");
		str = str.replace(/\\"/g, '\\"');
		str = str.replace(/\\&/g, "\\&");
		str = str.replace(/\\r/g, "\\r");
		str = str.replace(/\\t/g, "\\t");
		str = str.replace(/\\b/g, "\\b");
		str = str.replace(/\\f/g, "\\f");
		PGLog.write("After escapeSpecialSymbol : " + str);
		return str;
	}
	
	/**
	 * ユーザーデーター初期化（Debug機能）
	 */
	function initUserData(){
		PGConfig.remove('device_uuid');
		
		alert("ユーザーデーター初期化しました");
		
	}
	
	/**
	 * 接続環境切り替え（Debug機能）
	 */
	function changeConnectEnv(env_name){
		var switch_env_name = null;
		
		switch (env_name){
			 case 'suc_env_info':
			 	switch_env_name = "社内共通";
			 	break;
			 case 'dev_env_info':
			 	switch_env_name = "DEV環境";
			 	break;
			 case 'check_env_info':
			 	switch_env_name = "脆弱診断環境";
			 	break;
			 case 'staging_env_info':
			 	switch_env_name = "STAGING環境";
			 	break;
		}
		PG_Dialog.open_dialog(
			function(result)
			{
				// result : 1 -> はい, 2 -> いいえ
				if(result == 1){
					PGConfig.remove('device_uuid');
				}
				
				PGConfig.putStr('env_name',env_name);
				alert(switch_env_name + "に切り替えしました");
			},
			"はじめて"+ switch_env_name + "に接続しますか？",
			"アクセス環境切り替え",
			["はい","いいえ"]
		);
	}
	
})();

