/**
 * ブラウザのページ訪問履歴管理
 */
var AppPageHistory = (function() {

	var reload_url = null;
	var histories = new Array();

	// public Function
	return {
		getPageHistoryLength:function() {
			return getPageHistoryLength();
		},
		clearPageHistory:function() {
			return clearPageHistory();
		},
		pushPageHistory: function() {
	    	return pushPageHistory(AppPageHistory.reload_url);
	    },
		popPageHistory:function() {
			return popPageHistory();
		},
		savePageHistory:function() {
			return savePageHistory();
		},
		loadPageHistory:function() {
			return loadPageHistory();
		},
		setReloadUrl:function(url) {
			return setReloadUrl(url);
		},
		backHistory:function(fullReload) {
			return backHistory(fullReload);
		},
		reload:function() {
			return reload();
		},
	}

	/**
	 *	ページ訪問履歴件数を取得
	 */
	function getPageHistoryLength() {
		return histories.length;
	}


	/**
	 *	ページ訪問履歴をクリア
	 */
	function clearPageHistory() {
		histories = new Array();
		savePageHistory();
	}

	/**
	 *	ページ訪問履歴を1件追加
	 */
	function pushPageHistory(url) {

		// 未定義かnullなら保存しない(空文字列はルートなので保存する)
		if (typeof url === 'undefined' || url === null) {
			PGLog.write("AppPageHistory.pushPageHistory: url is undefined or null");
			return;
		}

		// yapp_homesは無条件で保存しない（初回登録でしか遷移しないため）
		if (url.match(/(?:yapp_homes|YappHomes)/i)) {
			return;
		}

		histories.push(url);

		// 最大数を超えたら消す
		if (histories.length > 10) {
			histories.shift();
		}

		savePageHistory();
	}

	/**
	 *	ページ訪問履歴を1件削除(戻るを押した)
	 */
	function popPageHistory() {
		var res = histories.pop();
		savePageHistory();
		return res;
	}

	/**
	 *	ページ訪問履歴を保存
	 */
	function savePageHistory() {

		var _key = "page_history";

		var save_data = JSON.stringify(histories);
		PGLog.write("AppPageHistory.savePageHistory: save_data:" + save_data);
		PGLocalStorage.setVal(_key, save_data);
	}

	/**
	 *	ページ訪問履歴を復元
	 */
	function loadPageHistory() {

		var _key = "page_history";

		try {
			var save_data = PGLocalStorage.getVal(_key);
			histories = JSON.parse(save_data);
			// 変なデータがあったら、はまらないように一旦クリア
			if (!(histories instanceof Array)) {
				clearPageHistory();
				return;
			}

		} catch (e) {
			clearPageHistory();
			return;
		}
	}

	function setReloadUrl(url) {
		AppPageHistory.reload_url = url;
		PGLog.write("AppPageHistory.setReloadUrl:" + AppPageHistory.reload_url);
	}

	function backHistory(fullReload) {

		if (typeof fullReload !== "boolean") fullReload = false;

		Popup.window_close();		// バックキー・リロード時は表示中に関係なくポップアップウィンドウを消すようにする

		var prev_url = AppPageHistory.popPageHistory();
		if (typeof prev_url === 'undefined' ) {
			PGLog.write("AppPageHistory.backHistory: prev_url is undefined");
			return;
		}
		var axi_server_url = PGLocalStorage.getVal('axi_server_url');
		if ((typeof axi_server_url === 'undefined') || axi_server_url === '' ) {
			PGLog.write("AppPageHistory.backHistory: axi_server_url is undefined");
			return;
		}
		var t = (new Date).getTime();
		var jump_url = axi_server_url + prev_url;
		if (fullReload) {
			// フルリロード遷移
			var boot_url = rawurlencode(jump_url);
			PGLog.write("AppPageHistory.backHistory: boot_url:" + boot_url);
			location.href = "index.html?boot_url="+ boot_url;
		} else {
			// 通常遷移
			if (jump_url.indexOf("?") === -1) {
				jump_url += "?t=" + t;
			} else {
				jump_url += "&t=" + t;
			}
			PGLog.write("AppPageHistory.backHistory: jump_url:" + jump_url);
			Popup.next_jump(jump_url, false);
		}
	}

	function reload() {

		Popup.window_close();		// バックキー・リロード時は表示中に関係なくポップアップウィンドウを消すようにする

		if (typeof AppPageHistory.reload_url === 'undefined' ) {
			PGLog.write("AppPageHistory.reload: reload_url is undefined");
			return;
		}
		var axi_server_url = PGLocalStorage.getVal('axi_server_url');
		if ((typeof axi_server_url === 'undefined') || axi_server_url === '' ) {
			PGLog.write("AppPageHistory.reload: axi_server_url is undefined");
			return;
		}

		var t = (new Date).getTime();
		var jump_url = axi_server_url + AppPageHistory.reload_url;
		if (jump_url.indexOf("?") === -1) {
			jump_url += "?reload=true&t=" + t;
		} else {
			jump_url += "&reload=true&t=" + t;
		}
		PGLog.write("AppPageHistory.reload: jump_url:" + jump_url);
		Popup.next_jump(jump_url, false);
	}
})();
