/**
 * 暗号・複合ユーティリティ
 *
 * @constructor
 */
var DecodeEncoder = function () {
	this.baseSalt = "LeZ(aKDkQyJ$";
	this.keyPassphrase = 'keyPassphrase';
	this.storagePassphrase = 'storagePassphrase';
};

/**
 * 初期化
 *
 * @param successChain 初期化完了後に実行したいcallback関数
 */
DecodeEncoder.prototype.init = function(successChain) {
	if(typeof successChain === 'undefined') {
		successChain = function(){};		// 未定義ならなにもしない
	}

	var self = this;
	PGConfig.getPassphrase(function(data){
		var key = data[self.keyPassphrase];
		var phrase = data[self.storagePassphrase];
		self.setKey(key);
		self.setPhrase(phrase);
		successChain();
	});
};

DecodeEncoder.prototype.easySort = function(value, num) {
	var result = value;
	if (typeof data === 'string') {
		var list = value.split('');
		var target = 0;
		var tmp = "";
		for(var i = 0; i < list.length; i++) {
			if (0 < i && i % num == 0) {
				target = i - 1;
				tmp = list[target];
				list[target] = list[i];
				list[i] = tmp;
			}
		}
		result = list.join('');
	}
	return result;
};

DecodeEncoder.prototype.createSalt = function(type) {
	var appendWord = "Nk#FMk%V+g*&";
	var baseSalt = CryptoJS.SHA256(this.baseSalt);
	var salt = baseSalt.toString();
	if(typeof type === 'undefined') {
		var max = 4;
		var min = 1;
		type = Math.floor( Math.random() * (max - min + 1) ) + min;
	}

	// TODO ひとまず4パターン。できればもっとたくさん欲しい
	switch (type) {
		case 1:
			// 3の倍数文字を切り出して並び替え
			salt = this.easySort(salt, 3);
			break;
		case 2:
			// 5の倍数文字を切り出して並び替え
			salt = this.easySort(salt, 5);
			break;
		case 3:
			// 適当な文字を先頭に追加してハッシュ
			salt = CryptoJS.SHA256(appendWord + salt);
			salt = salt.toString();
			break;
		case 4:
			// 適当な文字を後部に追加してハッシュ
			salt = CryptoJS.SHA256(salt + appendWord);
			salt = salt.toString();
			break;
		default :
			break;
	}

	return {type: type, salt: salt};
};

DecodeEncoder.prototype.createKey = function(basePhrase, salt) {
	//return CryptoJS.PBKDF2(basePhrase, salt, { keySize: 256/32, iterations: 1000 });
	return CryptoJS.PBKDF2(basePhrase, salt, { keySize: 256/32, iterations: 1 });		// 1000 -> 10 ... JSでデータを読み書きするごとに都度1000回ストレッチングすると重すぎるため
};

DecodeEncoder.prototype.setKey = function(value) {
	return PGLocalStorage.setVal(this.keyPassphrase, value, true);
};

DecodeEncoder.prototype.getKey = function() {
	return PGLocalStorage.getVal(this.keyPassphrase, true);
};

DecodeEncoder.prototype.setPhrase = function(value) {
	var key = this.getKey();
	var encryptedJsonString = this.getEncryptedJsonString(value, key);
	return PGLocalStorage.setVal(this.storagePassphrase, encryptedJsonString, true);
};

DecodeEncoder.prototype.getPhrase = function() {
	var key = this.getKey();
	var encryptedJsonString = PGLocalStorage.getVal(this.storagePassphrase, true);
	return this.getDecryptedString(encryptedJsonString, key);
};

DecodeEncoder.prototype.encrypt = function(value, phrase) {
	if(typeof phrase === 'undefined') {
		phrase = this.getPhrase();
	}
	return CryptoJS.AES.encrypt(value, phrase);
};

DecodeEncoder.prototype.getEncryptedJsonString = function(value, phrase) {
	if(typeof phrase === 'undefined') {
		phrase = this.getPhrase();
	}
	var encodeValue = CryptoJS.enc.Utf8.parse(value);
	var keySaltJson = this.createSalt();
	var key = this.createKey(phrase, keySaltJson['salt']);
	var encrypted = this.encrypt(encodeValue, key.toString());
	var iv = encrypted.iv.toString();
	var salt = encrypted.salt.toString();
	var ciphertext = encrypted.ciphertext.toString(CryptoJS.enc.Base64);
	return JSON.stringify({iv: iv, salt: salt, ciphertext: ciphertext, type: keySaltJson['type']});
};

DecodeEncoder.prototype.decrypt = function(encrypted, phrase) {
	if(typeof phrase === 'undefined') {
		phrase = this.getPhrase();
	}
	return CryptoJS.AES.decrypt(encrypted, phrase);
};

DecodeEncoder.prototype.getDecryptedString = function(encryptedJsonString, phrase) {
	if(typeof phrase === 'undefined') {
		phrase = this.getPhrase();
	}
	var result = '';
	try {
		var encryptedJson = JSON.parse(encryptedJsonString);
		var keySaltJson = this.createSalt(encryptedJson['type']);
		var key = this.createKey(phrase, keySaltJson['salt']);
		var encrypted = CryptoJS.lib.CipherParams.create({
			ciphertext: CryptoJS.enc.Base64.parse(encryptedJson['ciphertext'])
		});
		encrypted.salt = CryptoJS.enc.Hex.parse(encryptedJson['salt']);
		encrypted.iv= CryptoJS.enc.Hex.parse(encryptedJson['iv']);
		var decrypted = this.decrypt(encrypted, key.toString());
		result = decrypted.toString(CryptoJS.enc.Utf8);
	} catch(e) {
		PGLog.write("getDecryptedString error : " + e.message + " / " + e.stack + " / " + encryptedJsonString);
		result = encryptedJsonString;
	}
	return result;
};

if (decEnc == null) {
	var decEnc = new DecodeEncoder();
}
