/*
 *	SharedPreferencesクラスで端末ローカルに設定情報保存
 *	http://developer.android.com/reference/android/content/SharedPreferences.html
 */
var PGConfig = (function () {
	// public function
	return {
		getStr: function (key, callback, disableEncode) {
			return getStr(key, callback, disableEncode);
		},
		findKeyInSharedPreferences: function (key, callback) {
			return findKeyInSharedPreferences(key, callback);
		},
		isExistKey: function (key, callback) {
			return isExistKey(key, callback);
		},
		putStr: function (key, val, disableEncode) {
			return putStr(key, val, disableEncode);
		},
		remove: function (key) {
			return remove(key);
		},
		getPassphrase: function (success, error) {
			return getPassphrase(success, error);
		}
	}

	function initConfig() {
		PGConfig.getStr("my_config", "background");
		PGConfig.getStr("my_config", "voice");
	}

	/* 	preferenceから情報取得
	 *	用途:
	 *		key	: preferenceキー
	 */
	function getStr(key, callback, disableEncode) {
		// 保存ファイル名
		var preferences_version = PGLocalStorage.getVal('preferences_version');
		cordova.exec(
			function (params) {
				// 成功の場合
				if (typeof disableEncode === 'undefined') {
					if (params != null) params = decEnc.getDecryptedString(params);
				}
				callback(params);
			},
			function (errs) {
				navigator.notification.alert("設定情報取得に失敗しました");
			},
			"config",
			"get",
			[preferences_version, key] // send params
		);
	}

	/* 	preferenceにキーの存在チェック
	 *	用途:
	 *		key	: preferenceキー
	 */
	function findKeyInSharedPreferences(key, callback) {
		// 保存ファイル名
		var preferences_version = PGLocalStorage.getVal('preferences_version');
		PGLog.write("preferences_version : " + preferences_version);

		cordova.exec(
			function (params) {
				// 成功の場合
				callback(params);
			},
			function (errs) {
				navigator.notification.alert("設定情報のキーが見つかりませんでした");
			},
			"config",
			"isExistKey",
			[preferences_version, key] // send params
		);
	}

	// 動作は、findKeyInSharedPreferences と同じです。（用途をはっきりさせたいために追加）
	function isExistKey(key, callback) {
		// 保存ファイル名
		var preferences_version = PGLocalStorage.getVal('preferences_version');
		cordova.exec(
			function (params) {
				// 成功の場合
				callback(params);
			},
			function (errs) {
				navigator.notification.alert("設定情報のキーが見つかりませんでした");
			},
			"config",
			"isExistKey",
			[preferences_version, key] // send params
		);
	}

	/*
	 *	preferenceにデーター保存
	 *	用途:
	 *		key	: preferenceキー
	 *		val	: 保存データー
	 */
	function putStr(key, val, disableEncode) {
		// 保存ファイル名
		var preferences_version = PGLocalStorage.getVal('preferences_version');
		if (typeof disableEncode === 'undefined') {
			if (val != null) val = decEnc.getEncryptedJsonString(val);
		}
		cordova.exec(
			function (params) {
				// callback success
			},
			function (errs) {
				navigator.notification.alert("設定情報保存に失敗しました");
			},
			"config",
			"put",
			[preferences_version, key, val]
		);
	}

	/*
	 *	preferenceにデーター保存(callbackあり)
	 *	用途:
	 *		key			: preferenceキー
	 *		val			: 保存データー
	 *		callback	: 終了時のコールバック(引数paramsはcordovaから直で渡される)
	 */
	function putStrWithCallback(key, val, callback, disableEncode) {
		// 保存ファイル名
		var preferences_version = PGLocalStorage.getVal('preferences_version');
		if (typeof disableEncode === 'undefined') {
			if (val != null) val = decEnc.getEncryptedJsonString(val);
		}
		cordova.exec(
			function (params) {
				// callback success
				callback(params);
			},
			function (errs) {
				navigator.notification.alert("設定情報保存に失敗しました");
			},
			"config",
			"put",
			[preferences_version, key, val]
		);
	}

	/* 	preferenceからデーター削除 
	 *	用途:
	 *		key	: preferenceキー
	 */
	function remove(key) {
		// 保存ファイル名
		var preferences_version = PGLocalStorage.getVal('preferences_version');
		cordova.exec(
			function (params) {
				// callback success
			},
			function (errs) {
				navigator.notification.alert("設定情報削除に失敗しました");
			},
			"config",
			"remove",
			[preferences_version, key]
		);
	}

	/**
	 * パスフレーズ情報を取得
	 *
	 * @param success
	 * @param error
	 */
	function getPassphrase(success, error) {
		if (typeof error === 'undefined') {
			error = function (error) {
				PGLog.write('get passphrase error: ' + error.toString());
				setTimeout(function(){
					PGLog.write('get passphrase retry start');
					PGConfig.getPassphrase(success, error);
				}, 100);		// 100ミリ秒待ってリトライ
			};
		}

		cordova.exec(success, error, "config", "getPassphrase", []);
	}

})();