/**
 *  * ローカルサーバプラグイン
 * <p>
 * CorHttpd プラグインを元に作成
 * </p>
 * @see https://github.com/floatinghotpot/cordova-httpd
 * @constructor
 */
var PGLocalHttpd = function () {
	// プロパティ
	this.enableLocalHttpd = false;
};

PGLocalHttpd.prototype.startServer = function(success, error) {
	if (!this.enableLocalHttpd) return error("localHttpd is disabled.");
	return cordova.exec(success, error, "localHttpd", "startServer", []);
};

PGLocalHttpd.prototype.stopServer = function(success, error) {
	if (!this.enableLocalHttpd) return error("localHttpd is disabled.");
	return cordova.exec(success, error, "localHttpd", "stopServer", []);
};

PGLocalHttpd.prototype.getURL = function(success, error) {
	if (!this.enableLocalHttpd) return error("localHttpd is disabled.");
	return cordova.exec(success, error, "localHttpd", "getURL", []);
};

PGLocalHttpd.prototype.getPort = function(success, error) {
	if(typeof error === 'undefined') {
		error = function(error){
			PGLog.write('port get error: '+error.toString());
		};
	}
	if (!this.enableLocalHttpd) return error("localHttpd is disabled.");
	return cordova.exec(success, error, "localHttpd", "getPort", []);
};

PGLocalHttpd.prototype.isRunning = function(success, error) {
	if(typeof error === 'undefined') {
		error = function(error){
			PGLog.write('isRunning: server is not running');
		};
	}
	if (!this.enableLocalHttpd) return error("localHttpd is disabled.");
	return cordova.exec(success, error, "localHttpd", "isRunning", []);
};

PGLocalHttpd.prototype.checkServerRunning = function(success, error) {
	PGLog.writeWithTime("PGLocalHttpd.checkServerRunning start");
	if(typeof success === 'undefined') {
		success = function(url){
			PGLog.write('check and startServer success: '+url);
		};
	}
	if(typeof error === 'undefined') {
		error = function(error){
			PGLog.write('check and startServer error: '+error.toString());
		};
	}
	if (!this.enableLocalHttpd) {
		PGLog.writeWithTime("PGLocalHttpd.checkServerRunning end disabled");
		return error("localHttpd is disabled.");
	}
	var _self = this;
	return cordova.exec(function(msg){
		_self.getURL(success, error);
		PGLog.writeWithTime("PGLocalHttpd.checkServerRunning end success");
	}, function(msg){
		// 再起動
		PGLog.write('server is stopping');
		_self.startServer(success, error);
		PGLog.writeWithTime("PGLocalHttpd.checkServerRunning end error");
	}, "localHttpd", "isRunning", []);
};

PGLocalHttpd.prototype.removeAllCache = function() {
	return cordova.exec(function(msg){}, function(error){}, "localHttpd", "removeAllCache", []);
};

PGLocalHttpd.prototype.loadResources = function(resources) {
	var _self = this;
	// ローカルサーバ起動中のみ実行
	this.checkServerRunning(
		function(url){
			PGLog.write('loadResources before check success: '+url);
			var target = '';
			for(var i = 0; i < resources.length; i++) {
				target = resources[i];
				target = _self.replaceLocalHost(target, url);
				jQuery.get(target);
			}

		}
	);
};

PGLocalHttpd.prototype.getDataBeforeCheckServer = function(pageData, callback) {
	PGLog.writeWithTime("PGLocalHttpd.getDataBeforeCheckServer start");
	var _self = this;
	this.checkServerRunning(
		function(url){
			PGLog.write('get page before check success: '+url);
			callback(_self.replaceLocalHost(pageData, url));
			PGLog.writeWithTime("PGLocalHttpd.getDataBeforeCheckServer end success");
		},
		function(error){
			PGLog.write('get page before check error: '+error.toString());
/*
			var axi_resource_url = PGLocalStorage.getVal('axi_resource_url');
			callback(_self.replaceLocalHost(pageData, axi_resource_url));
*/
			callback(pageData);
			PGLog.writeWithTime("PGLocalHttpd.getDataBeforeCheckServer end error");
		}
	);
};

PGLocalHttpd.prototype.replaceLocalHost = function(data, replaceUrl, target) {
	if (typeof target === "undefined") target = "http://localhost/";
	if (typeof data === 'string') {
		//var regexp = /http:\/\/localhost\//g;
		var regexp = new RegExp(target, "g");
		replaceUrl = replaceUrl + '/';
		data = data.replace(regexp, replaceUrl);
	}
	return data;
};

var pGLocalHttpd = new PGLocalHttpd();
