/**
 * 音声再生プラグイン(custom.audio.media)wrapperクラス
 * @ref http://developer.android.com/reference/android/media/SoundPool.html
 * @ref http://developer.android.com/reference/android/media/MediaPlayer.html
 * @type {{preload: Function, play: Function, loop: Function, stop: Function, unload: Function}}
 */
var PGMedia = (function() {
	return {
		preload: function (id, assetPath) {
			return preload(id, assetPath);
		},
		play: function (id) {
			return play(id);
		},
		loop: function (id) {
			return loop(id);
		},
		playForResume: function (id) {
			return playForResume(id);
		},
		pause: function (id) {
			return pause(id);
		},
		stop: function (id) {
			return stop(id);
		},
		unload: function (id) {
			return unload(id);
		}
	}
	
	function initMedia() {
		var bg_id = "background";
		var bg_path = "bgm/bgm_battle.m4a";
		PGMedia.preload(bg_id, bg_path);
	
		var sound_id = "punch";
		var sound_path = "bgm/punch.mp3";
		PGMedia.preload(sound_id, sound_path);
	}
	
	/* 
	 *	メモリに音声ファイルを入れておく
	 *	用途:
	 *		id			: audio id
	 *		assetPath	: ファイルパス
	 */
	function preload(id, assetPath, type) {
		if(typeof type === 'undefined') type = 'music';
		cordova.exec(
			function (params) {
				// no need to return
			},
			function (errs) {
				PGLog.write("Fail to preload audio file");
			},
			"media",
			"preload",
			[id, assetPath, type] // send params
		);
	}
	
	/* 
	 *	音声再生 (no loop)
	 *	用途:
	 *		id : audio id
	 */
	function play(id) {
		cordova.exec(
			function (params) {
				// no need to return
			},
			function (errs) {
				PGLog.write("Fail to play audio");
				// alert("can not play audio");	// debug
			},
			"media",
			"play",
			[id]
		);
	}
	
	/* 
	 *	音声再生 (loop)
	 *	用途:
	 *		id : audio id
	 */
	function loop(id) {
		cordova.exec(
			function (params) {
				// no need to return
			},
			function (errs) {
				PGLog.write("Fail to loop audio");
				//alert("can not loop audio");	// debug
			},
			"media",
			"loop",
			[id]
		);
	}
	
	/*
	 *	音声再生 (playForResume)
	 *	用途:
	 *		id : audio id
	 */
	function playForResume(id) {
		cordova.exec(
			function (params) {
				// no need to return
			},
			function (errs) {
				PGLog.write("Fail to playForResume audio");
				//alert("can not loop audio");	// debug
			},
			"media",
			"playForResume",
			[id]
		);
	}
	
	/*
	 *	音声再生停止
	 *	用途:
	 *		id : audio id
	 */
	function stop(id) {
		cordova.exec(
			function (params) {
				// no need to return
			},
			function (errs) {
				PGLog.write("Fail to stop audio");
				// alert("can not stop audio");	// debug
			},
			"media",
			"stop",
			[id]
		);
	}
	
	/**
	 * 音声一時停止
	 * @param id
	 */
	function pause(id) {
		cordova.exec(
			function (params) {
				// no need to return
			},
			function (errs) {
				PGLog.write("Fail to pause audio");
				// alert("can not stop audio");	// debug
			},
			"media",
			"pause",
			[id]
		);
	}
	
	/*
	 *	メモリから音声ファイルをリリースする
	 *	用途:
	 *		id : audio id
	 */
	function unload(id) {
		cordova.exec(
			function (params) {
				// no need to return
			},
			function (errs) {
				PGLog.write("Fail to unload audio");
			},
			"media",
			"unload",
			[id]
		);
	}	
	
})();
