/*
 *	課金機能実装
 */
var PGPurchase = (function () {
	// private field
	var store_product_info = null;
	var sdk_product_id = null;

	// なんらかの原因で重複呼ばれるときの対策
	var func_called_cnt = 0;

	// 課金プラグインの初期化リトライ管理
	var init_plugin_retry_count_max = 3;
	var init_plugin_retry_count = 0;

	// public function
	return {
		checkUserBirth: function () {
			return checkUserBirth();
		},
		checkUserBirthCallback: function (callback) {
			return checkUserBirthCallback(callback);
		},
		updateUserBirthInfo: function (birth_year, birth_month) {
			return updateUserBirthInfo(birth_year, birth_month);
		},
		updateUserBirthCallback: function (callback) {
			return updateUserBirthCallback(callback);
		},
		getStoreInfo: function () {
			return getStoreInfo();
		},
		initBillingPlugin: function () {
			return initBillingPlugin();
		},
		getValidProduct: function (callback) {
			return getValidProduct(callback);
		},
		displayShopPage: function (callback) {
			return displayShopPage(callback);
		},
		startPaymentProccess: function (product_id) {
			return startPaymentProccess(product_id);
		},
		createCoinDeposit: function (product_id) {
			return createCoinDeposit(product_id);
		},
		storePaymentProccess: function (callback) {
			return storePaymentProccess(callback);
		},
		storePaymentFinish: function (callback) {
			return storePaymentFinish(callback);
		},
		getPreviousUnfinishPaymentData: function () {
			return getPreviousUnfinishPaymentData();
		},
		getUnfinishPaymentData: function (callback) {
			return getUnfinishPaymentData(callback);
		},
		updateCoinDepositFinish: function () {
			return updateCoinDepositFinish();
		}
	}

	/**
	 * 購入者の年齢確認
	 */
	function checkUserBirth() {
		// インジケーター表示
		Indicator.beginLoadPage();

		init_plugin_retry_count = 0;
		func_called_cnt = 0;

		var post_data = {};
		var axi_server_url = PGLocalStorage.getVal('axi_server_url');
		var request_url = axi_server_url + "/YappShops/checkUserBirthInfo";

		ServerInquire.postData(request_url, post_data, PGPurchase.checkUserBirthCallback);
	}

	/**
	 * 購入者の年齢確認後
	 */
	function checkUserBirthCallback(callback) {
		var status = null;
		var response = null;
		var callback_from = null;

		if (typeof callback == 'object') {
			status = callback.status;
			response = callback.response;
			callback_from = callback.callback_from;

		} else {
			var callbackJson;
			try {
				callbackJson = JSON.parse(callback);
				status = callbackJson.status;
				response = callbackJson.response;
				callback_from = callbackJson.callback_from;
			} catch (e) {
				PGLog.write("callbackJson is not json:" + callback);
				PG_Dialog.dialogUserAgeCertificationFail();
			}
		}

		if (response == 'success') {
			if (status == 'ADULT') {
				PGLog.write("checkUserBirthCallback success");

				// 既に登録した場合はプロダクトリスト表示
				PGLocalStorage.setVal('callback_from', callback_from);

				// プロダクト情報取得 　->　購入画面表示
				PGPurchase.getStoreInfo();

				// インジケーター表示
				// Indicator.beginLoadPage();
			} else {
				Indicator.finishLoadPage();
				YAppShop.btnChargeGoldBeginClick();
			}
		} else {
			// もう一度サーバーに問い合わせ
			Indicator.finishLoadPage();
			PGPurchase.checkUserBirth();
		}
	}

	/**
	 * 購入者の年齢登録 (Bridgeユーザーデーター更新)
	 */
	function updateUserBirthInfo(birth_year, birth_month) {
		// 送信データー
		var post_data = {'year': birth_year, 'month': birth_month};

		var axi_server_url = PGLocalStorage.getVal('axi_server_url');
		var request_url = axi_server_url + "/YappShops/updateUserBirthInfo";

		ServerInquire.postData(request_url, post_data, PGPurchase.updateUserBirthCallback);
	}

	/**
	 * 購入者の年齢登録
	 */
	function updateUserBirthCallback(callback) {
		var response = null;
		var callback_from = null;

		if (typeof callback === 'object') {
			response = callback.response;
			callback_from = callback.callback_from;

		} else {
			var callbackJson;
			try {
				callbackJson = JSON.parse(callback);
				response = callbackJson.response;
				callback_from = callbackJson.callback_from;
			} catch (e) {
				PGLog.write("callbackJson is not json:" + callback);
				//YAppShop.btnChargeGoldBeginClick();
				YAppShop.btnChargeGoldError();
				return;
			}
		}

		if (response == 'success') {
			// 既に登録した場合はプロダクトリスト表示
			PGLocalStorage.setVal('callback_from', callback_from);

			// プロダクト情報取得 　->　購入画面表示
			PGPurchase.getStoreInfo();

			// インジケーター表示
			Indicator.beginLoadPage();
		} else {
			//YAppShop.btnChargeGoldBeginClick(); // 登録エラーの際に何もしない
			PG_Dialog.dialogUserAgeCertificationFail();		// 通常ではありえないはず。
		}
	}

	/**
	 * サーバーにプロダクト情報を問い合わせする
	 */
	function getStoreInfo() {
		var device_platform = device.platform;

		if (device_platform == 'Android') {
			// Android端末
			var payment_environment = PGLocalStorage.getVal('payment_environment');
			PGLog.write("getStoreInfo called");

			if (func_called_cnt < 1) {
				func_called_cnt = func_called_cnt + 1;
				cordova.exec(
					function () {
						func_called_cnt = 0;

						var post_data = {};
						var axi_server_url = PGLocalStorage.getVal('axi_server_url');
						var request_url = axi_server_url + "/YappShops/getStoreInfo";

						ServerInquire.postData(request_url, post_data, PGPurchase.getValidProduct);

					},
					function (errs) {
						func_called_cnt = 0;
						init_plugin_retry_count++;
						if (init_plugin_retry_count_max < init_plugin_retry_count) {
							PGLog.write("Fail getStoreInfo:" + errs);
							PG_Dialog.dialogInitBillingPluginFail();
							return;
						}

						navigator.notification.alert("プラグイン初期化が失敗されました");

						// もう一度実行する
						PGPurchase.getStoreInfo();
					},
					"billing",
					"init_plugin",
					[payment_environment] // send params
				);
			}
		} else {
			// ios端末
			cordova.exec(
				function () {
					PGPurchase.initBillingPlugin();
				},
				function (errs) {
					navigator.notification.alert("iTunes storeサーバーの接続に失敗しました");
				},
				"billing",
				"init_plugin",
				[] // send params
			);
		}
	}

	/**
	 * 【ios版】プラグインなど初期化
	 */
	function initBillingPlugin() {
		var post_data = {};
		var axi_server_url = PGLocalStorage.getVal('axi_server_url');
		var request_url = axi_server_url + "/YappShops/getStoreInfo";

		ServerInquire.postData(request_url, post_data, PGPurchase.getValidProduct);
	}

	/**
	 * Apple/Googleに販売情報を問い合わせし、AXIサーバーに記録する
	 */
	function getValidProduct(callback) {
		var response = null;

		if (typeof callback === 'object') {
			response = callback.response;
		} else {
			var callbackJson;
			try {
				callbackJson = JSON.parse(callback);
			} catch (e) {
				PGLog.write("callbackJson is not json:" + callback);

				// もう一度サーバーにプロダクト情報を問い合わせする
				PGPurchase.getStoreInfo();
				return ;
			}

			response = callbackJson.response;
		}

		if (response == 'success') {
			PGLog.write("getValidProduct callback");
			// Indicator.finishLoadPage();

			// リトライ情報を削除する
			if (PGLocalStorage.getVal('init_shop_retry_cnt') !== null) {
				PGLocalStorage.remove('init_shop_retry_cnt');
			}

			// サーバーからのレスポンス
			var store_info = callback.storeInfo;

			// 問い合わせ情報準備
			var products_id = [];	// プロダクトID配列

			var device_platform = device.platform;
			if (device_platform == 'Android') {
				// Android端末
				var cnt = 0;
				for (var arr_key in store_info) {
					var product_info = store_info[arr_key];
					products_id[cnt] = arr_key;
					cnt++;
				}

				// 変換後のプロダクトID配列
				PGLog.write("Googleに送信プロダクトリスト：" + JSON.stringify(products_id));
			} else {
				var id_arr = new Array();

				var cnt = 0;
				for (var arr_key in store_info) {
					id_arr[cnt] = arr_key;
					cnt++;
				}
				products_id[0] = id_arr;

			}

			// Apple/Googleに有効プロダクト情報を問い合わせする
			if (func_called_cnt < 1) {
				func_called_cnt = func_called_cnt + 1;
				cordova.exec(
					function (product_list) {
						// Apple/Googleからのレスポンス
						PGLog.write("Apple/Googleからのレスポンス");
						func_called_cnt = 0;

						if (device_platform == 'Android') {
							// Android端末
							PGLog.write(JSON.stringify(product_list));

							if (product_list.hasOwnProperty('inapp_purchase_data')) {
								// Jsonの中にorderIdが入ってたら、プロダクト消費したあとの情報 → 前回の支払いは正常に完了しなかった → Bridgeに入金処理が必要

								PGLog.write("has inapp_purchase_data in string");

								var axi_server_url = PGLocalStorage.getVal('axi_server_url');
								var request_url = axi_server_url + "/YappShops/updateAndroidCoinDeposit";
								ServerInquire.postData(request_url, product_list, PGPurchase.storePaymentFinish);

								// インジケータを非表示にする
								// Indicator.finishLoadPage();

							} else { // 正常のケース → プロダクト一覧表示
								PGLog.write("not exist inapp_purchase_data in string");

								store_product_info = product_list;

								// AXIサーバーに有効なプロダクト情報を渡し、記録する
								var post_data = {'product_list': product_list};

								var axi_server_url = PGLocalStorage.getVal('axi_server_url');
								var request_url = axi_server_url + "/YappShops/getValidProduct";
								ServerInquire.postData(request_url, post_data, PGPurchase.displayShopPage);
							}

						} else {
							// ios端末
							PGLog.write(JSON.stringify(product_list[0]));

							// AXIサーバーに有効なプロダクト情報を渡し、記録する
							var post_data = {'product_list': product_list[0]};

							var axi_server_url = PGLocalStorage.getVal('axi_server_url');
							var request_url = axi_server_url + "/YappShops/getValidProduct";
							ServerInquire.postData(request_url, post_data, PGPurchase.displayShopPage);
						}

					},
					function (errs) {
						func_called_cnt = 0;

						// プラグンン初期化が失敗
						PGLog.write("Fail to get product info from Apple/Google.");

						init_plugin_retry_count++;
						if (init_plugin_retry_count_max < init_plugin_retry_count) {
							PGLog.write("Fail getStoreInfo:" + errs);
							Indicator.finishLoadPage();
							PG_Dialog.dialogInitBillingPluginFail();
							return;
						}

						var device_platform = device.platform;
						if (device_platform == 'Android') {

							PGLog.write("もう一度Playストアに問い合わせします。");
							PGPurchase.getValidProduct(store_info);
						} else {
							navigator.notification.alert("iTunes storeサーバーの接続に失敗しました");
						}
					},
					"billing",
					"init_shop",
					products_id // send params
				);
			}
		} else {
			// リトライ回数変数
			var init_shop_retry_cnt;
			// Indicator.finishLoadPage();

			// ローカルストレージからリトライ回数を取得する
			if (PGLocalStorage.getVal('init_shop_retry_cnt') === null) {
				init_shop_retry_cnt = 0;
			} else {
				init_shop_retry_cnt = PGLocalStorage.getVal('init_shop_retry_cnt');
			}

			// リトライ回数をカウントする
			init_shop_retry_cnt = parseInt(init_shop_retry_cnt) + 1;

			// ローカルストレージに保存する
			PGLocalStorage.setVal('init_shop_retry_cnt', init_shop_retry_cnt);

			// 3回以上の場合は、ユーザーに通知する
			if (init_shop_retry_cnt > 3) {
				//navigator.notification.alert("通信状況の良いところで再度お試しください");
				PG_Dialog.dialogGetProductInfoFail();
				return;
			} else {
				// もう一度サーバーにプロダクト情報を問い合わせする
				PGPurchase.getStoreInfo();
			}
		}
	}

	/*
	 * 購入画面表示
	 */
	function displayShopPage(callback) {
		var response = null;

		if (typeof callback === 'object') {
			response = callback.response;
		} else {
			var callbackJson;
			try {
				callbackJson = JSON.parse(callback);
			} catch (e) {
				PGLog.write("callbackJson is not json:" + callback);

				// サーバーからレスポンスがないと、もう一度問い合わせを行う
				var post_data = {'product_list': store_product_info[0]};

				var axi_server_url = PGLocalStorage.getVal('axi_server_url');
				var request_url = axi_server_url + "/YappShops/getValidProduct";
				ServerInquire.postData(request_url, post_data, PGPurchase.displayShopPage);

				return;
			}

			response = callbackJson.response;
		}

		if (response == 'success') {
			PGLog.write("Display shop screen success");

			// 購入画面を立ち上げる
			var axi_server_url = PGLocalStorage.getVal('axi_server_url');
			var request_url = axi_server_url + "/YappShops/displayShop";

			AppContent.onLoadContent(request_url, null);

			// 前回の金購入で、入金処理が完了していない場合は、ここでやる
			var purchase_data_str = PGLocalStorage.getVal('payment_finish_retry_purchase_data');
			if ((typeof purchase_data_str !== 'undefined') && (purchase_data_str !== null)) {
				var purchase_data = JSON.parse(purchase_data_str); // stringからJsonに変換
				var axi_server_url = PGLocalStorage.getVal('axi_server_url');
				var request_url = axi_server_url + "/YappShops/updateAndroidCoinDeposit";
				ServerInquire.postData(request_url, purchase_data, PGPurchase.storePaymentFinish);
			}
		} else {
			// サーバーからレスポンスがないと、もう一度問い合わせを行う
			var post_data = {'product_list': store_product_info[0]};

			var axi_server_url = PGLocalStorage.getVal('axi_server_url');
			var request_url = axi_server_url + "/YappShops/getValidProduct";
			ServerInquire.postData(request_url, post_data, PGPurchase.displayShopPage);
		}
	}

	/**
	 * 購入処理開始
	 */
	function startPaymentProccess(product_id) {
		// 購入処理を呼び出し
		PGPurchase.createCoinDeposit(product_id);

		// Init sdk productId
		sdk_product_id = null;

		// インジケーター表示
		Indicator.beginLoadPage();
	}

	/**
	 * GooglePlay/Applestoreからの情報でBRIDGE側に仮想通貨付与情報作成
	 */
	function createCoinDeposit(product_id) {
		// ローカルストレージに保存する
		PGLocalStorage.setVal('purchase_retry_product_id', product_id);

		var device_platform = device.platform;
		if (device_platform == 'Android') {
			// Android端末
			device_type = 2;
		} else {
			// iOS端末
			device_type = 1;
		}

		PGLog.write("購入処理開始。プロダクトID : " + product_id);

		var post_data = {'product_id': product_id, 'device_type': device_type};
		var axi_server_url = PGLocalStorage.getVal('axi_server_url');
		var request_url = axi_server_url + "/YappShops/createCoinDeposit";
		ServerInquire.postData(request_url, post_data, PGPurchase.storePaymentProccess);
	}

	/**
	 *　ストア側の決済処理
	 */
	function storePaymentProccess(callback) {

		if ((PGLocalStorage.getVal('start_buy_coin') !== null) && (typeof callback === 'undefined')) {
			// resumeをを行うために、ローカルストレージに保存フラグを削除する
			PGLocalStorage.remove('start_buy_coin');

			// ログイン導線へ遷移
			App.onResume();
		} else {

			var response = null;
			var status = null;

			if (typeof callback === 'object') {
				response = callback.response;
				status = callback.status;

			} else {
				var callbackJson;
				try {
					callbackJson = JSON.parse(callback);
				} catch (e) {
					PGLog.write("callbackJson is not json:" + callback);

					// 単純に処理中止
					return;
				}

				response = callbackJson.response;
				status = callbackJson.status;
			}

			if ((response == "success") && (status == "underage")) {
				// 課金制限対応で、エラー画面立ち上げ
				var axi_server_url = PGLocalStorage.getVal('axi_server_url');
				var request_url = axi_server_url + "/YappShops/underAgeError";

				// エラーページ表示
				AppContent.onLoadContent(request_url, null);

			} else if ((response == "success") && (status == "payment")) {
				// リトライ情報を削除する
				PGLocalStorage.remove('purchase_retry_product_id');

				if (PGLocalStorage.getVal('purchase_retry_cnt') !== null) {
					PGLocalStorage.remove('purchase_retry_cnt');
				}

				if (PGLocalStorage.getVal('payment_screen_load_cnt') !== null) {
					PGLocalStorage.remove('payment_screen_load_cnt');
				}

				// ローカルストレージに購入フラグを保存する (resume処理を飛ばすため)
				PGLocalStorage.setVal('start_buy_coin', 'start');

				var product_id = callback.product_id;
				sdk_product_id = callback.product_id; // metaps sdk
				PGLog.write("購入プロダクトID:" + product_id);

				if (func_called_cnt < 1) {
					func_called_cnt = func_called_cnt + 1;
					cordova.exec(
						function (purchase_data) {
							func_called_cnt = 0;

							// 成功の場合はすぐに  、購入アイテム消費（Googleplay仕様->Androidの特別処理）
							var device_platform = device.platform;
							if (device_platform == 'Android') {
								// 成功の場合は、Bridge側に入金処理開始
								PGLog.write("purchase_data after consume:" + JSON.stringify(purchase_data));

								// Googleplayで決済情報データーをローカルストレージに保存する
								PGLocalStorage.setVal('payment_finish_retry_purchase_data', JSON.stringify(purchase_data));

								// ゲームサーバーにリクエストする
								var axi_server_url = PGLocalStorage.getVal('axi_server_url');
								var request_url = axi_server_url + "/YappShops/updateAndroidCoinDeposit";
								ServerInquire.postData(request_url, purchase_data, PGPurchase.storePaymentFinish);
							} else {
								// ios版
							}
						},
						function (errs) {
							func_called_cnt = 0;

							// 購入失敗
							PGLog.write("Fail to connect to googleplay/iTune store");

							var device_platform = device.platform;
							if (device_platform == 'Android') {

								PGLocalStorage.remove('callback_from');
								PGLocalStorage.remove('payment_finish_retry_purchase_data');

								if (PGLocalStorage.getVal('payment_finish_retry_cnt') !== null) {
									PGLocalStorage.remove('payment_finish_retry_cnt');
								}

							} else {
								// ios版 
							}

							// インジケータを非表示にする
							Indicator.finishLoadPage();
						},
						"billing",
						"purchase",
						[product_id] // send params
					);
				}
			} else {
				// リトライ回数変数
				var purchase_retry_cnt;

				// ローカルストレージからプロダクトIDを取得する
				var product_id = PGLocalStorage.getVal('purchase_retry_product_id');

				// ローカルストレージからリトライ回数を取得する
				if (PGLocalStorage.getVal('purchase_retry_cnt') === null) {
					purchase_retry_cnt = 0;
				} else {
					purchase_retry_cnt = PGLocalStorage.getVal('purchase_retry_cnt');
				}

				// リトライ回数をカウントする
				purchase_retry_cnt = parseInt(purchase_retry_cnt) + 1;

				// ローカルストレージに保存する
				PGLocalStorage.setVal('purchase_retry_cnt', purchase_retry_cnt);

				// 3回以上の場合は、ユーザーに通知する
				if (purchase_retry_cnt > 3) {
					navigator.notification.alert("通信状況の良いところで再度お試しください");

					// インジケータ非表示
					Indicator.finishLoadPage();
				} else {
					// もう一度サーバーに仮想通貨付与情報作成を依頼する
					PGPurchase.createCoinDeposit(product_id);
				}
			}
		}
	}

	/**
	 * 購入処理終了
	 */
	function storePaymentFinish(callback) {
		var response = null;

		if (typeof callback === 'object') {
			response = callback.response;
		} else {
			var callbackJson;
			try {
				callbackJson = JSON.parse(callback);
			} catch (e) {
				PGLog.write("callbackJson is not json:" + callback);

				// インジケータ非表示
				Indicator.finishLoadPage();

				// 金購入画面をリロード
				PGPurchase.getStoreInfo();
				return ;
			}

			response = callbackJson.response;
		}

		if (response == "success") {
			// 決済処理が完了
			PGLog.write("payment process finish.");

			var axi_server_url = PGLocalStorage.getVal('axi_server_url');
			var callback_from = PGLocalStorage.getVal('callback_from');

			// リトライ情報を削除する
			PGLocalStorage.remove('callback_from');
			PGLocalStorage.remove('payment_finish_retry_purchase_data');

			if (PGLocalStorage.getVal('payment_finish_retry_cnt') !== null) {
				PGLocalStorage.remove('payment_finish_retry_cnt');
			}

			if (PGLocalStorage.getVal('payment_screen_load_cnt') !== null) {
				PGLocalStorage.remove('payment_screen_load_cnt');
			}

			// 金販売画面をリロードする
			var request_url = axi_server_url + "/YappShops/displayShop";
			AppContent.onLoadContent(request_url, null);

			var device_platform = device.platform;
			if (device_platform == 'iOS') {
				// ペイメント完了メッセージは別途対応が必要です
				navigator.notification.alert(callback.message, callbackFunc, 'App内課金を入手');
			}

			// Track purchase event
			PGLog.write("********** TrackPurchaselEvent ************");
			MetapsAnalytics.trackPurchaselEvent(sdk_product_id);

			// インジケータ非表示
			Indicator.finishLoadPage();
		} else if (callback.response === "skip") {
			// インジケータ非表示
			Indicator.finishLoadPage();

			var payment_screen_load_cnt = PGLocalStorage.getVal('payment_screen_load_cnt');
			if (payment_screen_load_cnt != 1) {
				payment_screen_load_cnt = 1;

				// 購入キャンセルのとき、１回のみ購入画面をリロードする
				PGLocalStorage.setVal('payment_screen_load_cnt', payment_screen_load_cnt);

				// 金購入画面をリロード
				PGPurchase.getStoreInfo();
			} else {
				PG_Dialog.dialogUserAgeCertificationFail();		// 仮
			}

		} else {
			// リトライ回数変数
			var payment_finish_retry_cnt;

			// ローカルストレージからリトライ回数を取得する
			if (PGLocalStorage.getVal('payment_finish_retry_cnt') === null) {
				payment_finish_retry_cnt = 0;
			} else {
				payment_finish_retry_cnt = PGLocalStorage.getVal('payment_finish_retry_cnt');
			}

			// リトライ回数をカウントする
			payment_finish_retry_cnt = parseInt(payment_finish_retry_cnt) + 1;

			// ローカルストレージに保存する
			PGLocalStorage.setVal('payment_finish_retry_cnt', payment_finish_retry_cnt);

			// 3回以上の場合は、ユーザーに通知する
			if (payment_finish_retry_cnt > 3) {
				navigator.notification.alert("通信状況の良いところで再度お試しください");

				// インジケータ非表示
				Indicator.finishLoadPage();
			} else {
				// もう一度ゲームサーバーに入金をリクエストする
				var purchase_data_str = PGLocalStorage.getVal('payment_finish_retry_purchase_data');
				var purchase_data = JSON.parse(purchase_data_str); // stringからJsonに変換
				var axi_server_url = PGLocalStorage.getVal('axi_server_url');
				var request_url = axi_server_url + "/YappShops/updateAndroidCoinDeposit";
				ServerInquire.postData(request_url, purchase_data, PGPurchase.storePaymentFinish);
			}
		}
	}

	/**
	 * 前回の購入で未完成トランザクションデーター取得
	 */
	function getPreviousUnfinishPaymentData() {
		var func_called_cnt = 0;
		init_plugin_retry_count = 0;
		var payment_environment = PGLocalStorage.getVal('payment_environment');

		if (func_called_cnt < 1) {
			func_called_cnt = func_called_cnt + 1;
			cordova.exec(
				function () {
					func_called_cnt = 0;

					var post_data = {};
					var axi_server_url = PGLocalStorage.getVal('axi_server_url');
					var request_url = axi_server_url + "/YappShops/getStoreInfo";

					ServerInquire.postData(request_url, post_data, PGPurchase.getUnfinishPaymentData);

				},
				function (errs) {
					func_called_cnt = 0;
					PGLog.write("プラグイン初期化が失敗されました");
				},
				"billing",
				"init_plugin",
				[payment_environment] // send params
			);
		}
	}

	/**
	 * 未完成トランザクションのデーター取得
	 */
	function getUnfinishPaymentData(callback) {
		var response = null;
		var func_called_cnt = 0;

		if (typeof callback === 'object') {
			response = callback.response;
		} else {
			var callbackJson;
			try {
				callbackJson = JSON.parse(callback);
			} catch (e) {
				PGLog.write("callbackJson is not json:" + callback);

				// もう一度サーバーにプロダクト情報を問い合わせする
				PGPurchase.getStoreInfo();
			}

			response = callbackJson.response;
		}

		if (response == 'success') {
			PGLog.write("getValidProduct callback");

			// リトライ情報を削除する
			if (PGLocalStorage.getVal('init_shop_retry_cnt') !== null) {
				PGLocalStorage.remove('init_shop_retry_cnt');
			}

			// サーバーからのレスポンス
			var store_info = callback.storeInfo;

			// 問い合わせ情報準備
			var products_id = [];	// プロダクトID配列

			var device_platform = device.platform;
			if (device_platform == 'Android') {
				// Android端末
				var cnt = 0;
				for (var arr_key in store_info) {
					var product_info = store_info[arr_key];
					products_id[cnt] = arr_key;
					cnt++;
				}

				// 変換後のプロダクトID配列
				PGLog.write("Googleに送信プロダクトリスト：" + JSON.stringify(products_id));
			} else {
				// iOS版
			}

			// Apple/Googleに有効プロダクト情報を問い合わせする
			if (func_called_cnt < 1) {
				func_called_cnt = func_called_cnt + 1;
				cordova.exec(
					function (product_list) {
						// Apple/Googleからのレスポンス
						PGLog.write("Apple/Googleからのレスポンス");
						func_called_cnt = 0;

						if (device_platform == 'Android') {
							// Android端末
							PGLog.write(JSON.stringify(product_list));

							if (product_list.hasOwnProperty('inapp_purchase_data')) {
								// Jsonの中にorderIdが入ってたら、プロダクト消費したあとの情報 → 前回の支払いは正常に完了しなかった → Bridgeに入金処理が必要

								PGLog.write("has inapp_purchase_data in string");

								var axi_server_url = PGLocalStorage.getVal('axi_server_url');
								var request_url = axi_server_url + "/YappShops/updateAndroidCoinDeposit";
								ServerInquire.postData(request_url, product_list, PGPurchase.updateCoinDepositFinish);

							}
						} else {
							// iOS版
						}
					},
					function (errs) {
						func_called_cnt = 0;

						// プラグンン初期化が失敗
						PGLog.write("Fail to get product info from Apple/Google.");

						init_plugin_retry_count++;
						if (init_plugin_retry_count_max < init_plugin_retry_count) {
							PGLog.write("Fail getUnfinishPaymentData:" + errs);
							Indicator.finishLoadPage();
							PG_Dialog.dialogInitBillingPluginFail();
							return;
						}

						var device_platform = device.platform;
						if (device_platform == 'Android') {

							PGLog.write("もう一度Playストアに問い合わせします。");
							PGPurchase.getValidProduct(store_info);
						} else {
							// iOS版
						}
					},
					"billing",
					"init_shop",
					products_id // send params
				);
			}
		}
	}

	/**
	 * ショップやくじページで所持金更新後のコールバック
	 */
	function updateCoinDepositFinish() {
		// なんもしなくても良い
	}

	/**
	 * 課金処理終了時のコールバック
	 */
	function callbackFunc() {
	}


})();
