/*
 * ゲームサーバーとやり取り機能
 */
var ServerInquire = (function () {
	var push_noti_repost_device_token = null;
	var push_noti_repost_push_state = null;
	var push_noti_repost_retry_cnt = 0;

	return {
		postData: function (request_url, post_data, callback, errCallback) {
			return postData(request_url, post_data, callback, errCallback);
		},
		postDataWithDataType: function (request_url, post_data, datatype, callback, errCallback) {
			return postDataWithDataType(request_url, post_data, datatype, callback, errCallback);
		},
		postDataWithDataTypeForPopup: function (request_url, post_data, datatype, callback, errCallback) {
			return postDataWithDataTypeForPopup(request_url, post_data, datatype, callback, errCallback);
		},
		postDataBySSL: function (request_url, post_data, callback, errCallback) {
			return postDataBySSL(request_url, post_data, callback, errCallback);
		},
		saveDeviceInfo: function (device_token, push_state) {
			return saveDeviceInfo(device_token, push_state);
		}
	}

	/*
	 * サーバーにデーターをPOSTする
	 */
	function postData(request_url, post_data, callback, errCallback) {
		var t = (new Date).getTime();
		var request_url = request_url + "?t=" + t;
		if (typeof errCallback !== "function") errCallback = AppContent.contentLoadErr;

		PGLog.write("postData: sen_post_to:" + request_url);

		// リクエスト送信する
		jQuery.ajax({
			type: "POST",
			url: request_url,
			cache: false,
			/*dataType: 'JSON',*/
			data: post_data,
			success: function (data, status, xhr) {
				//callback(data);
				pGLocalHttpd.getDataBeforeCheckServer(data, callback);
			},
			error: function (xhr, status, error) {
				//エラーメッセージの表示
				PGLog.write("postData : Fail to post data to axi server. xhr:" + JSON.stringify(xhr) + ", status:" + status + ", error:" + error);
				errCallback(status, error);
			}
		});
	}

	/*
	 * サーバーにリクエストする。datatypeを指定できる版
	 */
	function postDataWithDataType(request_url, post_data, datatype, callback, errCallback) {
		var t = (new Date).getTime();
		var request_url = request_url + "?t=" + t;
		if (typeof errCallback !== "function") errCallback = AppContent.contentLoadErr;

		PGLog.write("postDataWithDataType: sen_post_to:" + request_url);

		// リクエスト送信する
		jQuery.ajax({
			type: "POST",
			url: request_url,
			cache: false,
			dataType: datatype,
			data: post_data,
			success: function (data, status, xhr) {
				//callback(data);
				pGLocalHttpd.getDataBeforeCheckServer(data, callback);
			},
			error: function (xhr, status, error) {
				//エラーメッセージの表示
				PGLog.write("postDataWithDataType : Fail to post data to axi server. xhr:" + JSON.stringify(xhr) + ", status:" + status + ", error:" + error);
				if ((xhr.status == 500 || xhr.status == 404 || xhr.status == 200) && typeof xhr.responseText !== 'undefined' && xhr.responseText.length > 0) {		// 200 は来ないはずだけど保険で設定
					// ゲームサーバが用意したエラーページに遷移させる
					pGLocalHttpd.getDataBeforeCheckServer(xhr.responseText, callback);
				} else {
					errCallback(status, error);
				}
			}
		});
	}

	/**
	 * データ送信
	 * ※ポップアップ用にレスポンスヘッダからメンテナンスかどうかを判別処理を加えてあります。
	 *
	 * @param request_url
	 * @param post_data
	 * @param datatype
	 * @param callback
	 * @param errCallback
	 */
	function postDataWithDataTypeForPopup(request_url, post_data, datatype, callback, errCallback) {
		var t = (new Date).getTime();
		var request_url = request_url + "?t=" + t;
		if (typeof errCallback !== "function") errCallback = AppContent.contentLoadErr;

		if (typeof datatype !== "string") datatype = "text";

		PGLog.write("postDataWithDataTypeForPopup: sen_post_to:" + request_url);

		// リクエスト送信する
		jQuery.ajax({
			type: "POST",
			url: request_url,
			cache: false,
			dataType: datatype,
			data: post_data,
			success: function (data, status, xhr) {
				// メンテチェック
				var isMaintenance = xhr.getResponseHeader('X-APP-MAINTENANCE');
				if (isMaintenance == 1) {
					errCallback(status, null, isMaintenance);
				} else {
					pGLocalHttpd.getDataBeforeCheckServer(data, callback);
				}
			},
			error: function (xhr, status, error) {
				//エラーメッセージの表示
				PGLog.write("postDataWithDataTypeForPopup : Fail to post data to axi server. xhr:" + JSON.stringify(xhr) + ", status:" + status + ", error:" + error);
				if ((xhr.status == 500 || xhr.status == 404 || xhr.status == 200) && typeof xhr.responseText !== 'undefined' && xhr.responseText.length > 0) {		// 200 は来ないはずだけど保険で設定
					// ゲームサーバが用意したエラーページに遷移させる
					pGLocalHttpd.getDataBeforeCheckServer(xhr.responseText, callback);
				} else {
					errCallback(status, error);
				}
			}
		});
	}

	/*
	 * SSLでサーバーに通信
	 */
	function postDataBySSL(request_url, post_data, callback, errCallback) {
		cordova.exec(
			function (params) {
				// 成功の場合
				callback(params);
			},
			function (error) {
				PGLog.write("Fail to post data to bridge server");
				errCallback(error);
			},
			"postData",
			"secure",
			[request_url, JSON.stringify(post_data)] // send params
		);
	}

	/*
	 * ゲームサーバーにデバイス情報を保存する
	 */
	function saveDeviceInfo(device_token, push_state) {
		var t = (new Date).getTime();
		var axi_server_url = PGLocalStorage.getVal('axi_server_url');

		// サーバーに再送信用のデーター
		push_noti_repost_device_token = device_token;
		push_noti_repost_push_state = push_state;

		// デバイスID
		var device_uuid = device.uuid;
		var device_model = device.model;
		var pf_type = device.platform;

		PGLog.write(device_uuid);
		PGLog.write(device_model);
		PGLog.write("pf_type:" + pf_type);
		PGLog.write("device_token:" + device_token);

		var post_data = {
			'pf_type': pf_type,
			'device_uuid': device_uuid,
			'device_model': device_model,
			'device_token': device_token,
			'push_state': push_state
		};

		var request_url = axi_server_url + "/YappPush/store_data_2_db?t=" + t;

		// リクエスト送信する
		jQuery.ajax({
			type: "POST",
			url: request_url,
			cache: false,
			/*dataType: 'JSON',*/
			data: post_data,
			success: function (data, status, xhr) {
				if ((data.response == 'success') && (data.status == 'ng') && (push_noti_repost_retry_cnt <= 3)) {
					// もう一度サーバーに送信
					ServerInquire.saveDeviceInfo(push_noti_repost_device_token, push_noti_repost_push_state);
					push_noti_repost_retry_cnt = push_noti_repost_retry_cnt + 1;
				}
			},
			error: function (xhr, status, error) {
				//エラーメッセージの表示
				PGLog.write("Push通知の内容はゲームサーバーに送信できませんでした。" + status + ":" + error);

				// もう一度サーバーに送信
				if (push_noti_repost_retry_cnt <= 3) {
					ServerInquire.saveDeviceInfo(push_noti_repost_device_token, push_noti_repost_push_state);
					push_noti_repost_retry_cnt = push_noti_repost_retry_cnt + 1;
				}
			}
		});

	}

})();

